@@ -1,71 +1,63 @@
 /* Load a task using the single server, and then run it
    as if we were the kernel.
-   Copyright (C) 1993,94,95,96,97,98,99,2000,01,02,2006
+   Copyright (C) 1993,94,95,96,97,98,99,2000,01,02,2006,14,16
      Free Software Foundation, Inc.
 
-This file is part of the GNU Hurd.
+   This file is part of the GNU Hurd.
 
-The GNU Hurd is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+   The GNU Hurd is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
 
-The GNU Hurd is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+   The GNU Hurd is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with the GNU Hurd; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   You should have received a copy of the GNU General Public License
+   along with the GNU Hurd; see the file COPYING.  If not, write to
+   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* Written by Michael I. Bushnell.  */
 
 #include <mach.h>
 #include <mach/notify.h>
 #include <device/device.h>
-#include <a.out.h>
 #include <mach/message.h>
 #include <mach/mig_errors.h>
+#include <mach/task_notify.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
 #include <pthread.h>
 #include <fcntl.h>
-#include <elf.h>
 #include <mach/mig_support.h>
 #include <mach/default_pager.h>
-#include <mach/machine/vm_param.h> /* For VM_XXX_ADDRESS */
 #include <argp.h>
 #include <hurd/store.h>
+#include <hurd/ihash.h>
+#include <sys/reboot.h>
 #include <sys/mman.h>
 #include <version.h>
 
 #include "notify_S.h"
-#include "ourdevice_S.h"
+#include "device_S.h"
 #include "io_S.h"
 #include "device_reply_U.h"
 #include "io_reply_U.h"
 #include "term_S.h"
 #include "bootstrap_S.h"
 /* #include "tioctl_S.h" */
+#include "mach_S.h"
+#include "mach_host_S.h"
+#include "gnumach_S.h"
+#include "task_notify_S.h"
 
 #include "boot_script.h"
 
 #include <hurd/auth.h>
 
-#ifdef UX
-#undef STORE			/* We can't use libstore when under UX.  */
-#else
-#define STORE
-#endif
-
-#ifdef UX
-
-#include "ux.h"
-
-#else  /* !UX */
-
 #include <unistd.h>
 #include <fcntl.h>
 #include <signal.h>
@@ -76,6 +68,14 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include <hurd.h>
 #include <assert.h>
 
+/* We support two modes of operation.  Traditionally, Subhurds were
+   privileged, i.e. they had the privileged kernel ports.  This has a
+   few drawbacks.  Privileged subhurds can manipulate all tasks on the
+   system and halt the system.  Nowadays we allow an unprivileged
+   mode.  */
+static int privileged;
+static int want_privileged;
+
 static struct termios orig_tty_state;
 static int isig;
 static char *kernel_command_line;
@@ -105,14 +105,23 @@ restore_termstate ()
 
 #define host_fstat fstat
 typedef struct stat host_stat_t;
-#define host_exit exit
 
-#endif /* UX */
+void __attribute__ ((__noreturn__))
+host_exit (int status)
+{
+  restore_termstate ();
+  exit (status);
+}
 
-mach_port_t privileged_host_port, master_device_port, defpager;
+mach_port_t privileged_host_port, master_device_port;
+mach_port_t pseudo_privileged_host_port;
 mach_port_t pseudo_master_device_port;
 mach_port_t receive_set;
-mach_port_t pseudo_console, pseudo_root;
+mach_port_t pseudo_console, pseudo_root, pseudo_time;
+mach_port_t pseudo_pset;
+task_t pseudo_kernel;
+mach_port_t task_notification_port;
+mach_port_t dead_task_notification_port;
 auth_t authserver;
 
 struct store *root_store;
@@ -145,284 +154,76 @@ void safe_gets (char *buf, int buf_len)
   fgets (buf, buf_len, stdin);
 }
 
-char *useropen_dir;
-
-int
-useropen (const char *name, int flags, int mode)
-{
-  if (useropen_dir)
-    {
-      static int dlen;
-      if (!dlen) dlen = strlen (useropen_dir);
-      {
-	int len = strlen (name);
-	char try[dlen + 1 + len + 1];
-	int fd;
-	memcpy (try, useropen_dir, dlen);
-	try[dlen] = '/';
-	memcpy (&try[dlen + 1], name, len + 1);
-	fd = open (try, flags, mode);
-	if (fd >= 0)
-	  return fd;
-      }
-    }
-  return open (name, flags, mode);
-}
+extern char *useropen_dir;
 
+/* XXX: glibc should provide mig_reply_setup but does not.  */
+/* Fill in default response.  */
+void
+mig_reply_setup (
+	const mach_msg_header_t	*in,
+	mach_msg_header_t	*out)
+{
+      static const mach_msg_type_t RetCodeType = {
+		/* msgt_name = */		MACH_MSG_TYPE_INTEGER_32,
+		/* msgt_size = */		32,
+		/* msgt_number = */		1,
+		/* msgt_inline = */		TRUE,
+		/* msgt_longform = */		FALSE,
+		/* msgt_deallocate = */		FALSE,
+		/* msgt_unused = */		0
+	};
+
+#define	InP	(in)
+#define	OutP	((mig_reply_header_t *) out)
+      OutP->Head.msgh_bits =
+	MACH_MSGH_BITS(MACH_MSGH_BITS_REMOTE(InP->msgh_bits), 0);
+      OutP->Head.msgh_size = sizeof *OutP;
+      OutP->Head.msgh_remote_port = InP->msgh_remote_port;
+      OutP->Head.msgh_local_port = MACH_PORT_NULL;
+      OutP->Head.msgh_seqno = 0;
+      OutP->Head.msgh_id = InP->msgh_id + 100;
+      OutP->RetCodeType = RetCodeType;
+      OutP->RetCode = MIG_BAD_ID;
+#undef InP
+#undef OutP
+}
+
 int
-request_server (mach_msg_header_t *inp,
-		mach_msg_header_t *outp)
+boot_demuxer (mach_msg_header_t *inp,
+	      mach_msg_header_t *outp)
 {
-  extern int io_server (mach_msg_header_t *, mach_msg_header_t *);
-  extern int device_server (mach_msg_header_t *, mach_msg_header_t *);
-  extern int notify_server (mach_msg_header_t *, mach_msg_header_t *);
-  extern int term_server (mach_msg_header_t *, mach_msg_header_t *);
-/*  extern int tioctl_server (mach_msg_header_t *, mach_msg_header_t *); */
-  extern int bootstrap_server (mach_msg_header_t *, mach_msg_header_t *);
-  extern void bootstrap_compat ();
-
-#if 0
-  if (inp->msgh_local_port == bootport && boot_like_cmudef)
+  mig_routine_t routine;
+  mig_reply_setup (inp, outp);
+  if ((routine = io_server_routine (inp)) ||
+      (routine = device_server_routine (inp)) ||
+      (routine = notify_server_routine (inp)) ||
+      (routine = term_server_routine (inp)) ||
+      (routine = mach_server_routine (inp)) ||
+      (routine = mach_host_server_routine (inp)) ||
+      (routine = gnumach_server_routine (inp)) ||
+      (routine = task_notify_server_routine (inp))
+      /* (routine = tioctl_server_routine (inp)) */)
     {
-      if (inp->msgh_id == 999999)
-	{
-	  bootstrap_compat (inp, outp);
-	  return 1;
-	}
-      else
-	return bootstrap_server (inp, outp);
+      (*routine) (inp, outp);
+      return TRUE;
     }
   else
-#endif
-   return (io_server (inp, outp)
-	   || device_server (inp, outp)
-	   || notify_server (inp, outp)
-	   || term_server (inp, outp)
-	   /*	    || tioctl_server (inp, outp) */);
+    return FALSE;
 }
 
-vm_address_t
-load_image (task_t t,
-	    char *file)
-{
-  int fd;
-  union
-    {
-      struct exec a;
-      Elf32_Ehdr e;
-    } hdr;
-  char msg[] = ": cannot open bootstrap file\n";
-
-  fd = useropen (file, O_RDONLY, 0);
-
-  if (fd == -1)
-    {
-      write (2, file, strlen (file));
-      write (2, msg, sizeof msg - 1);
-      task_terminate (t);
-      host_exit (1);
-    }
-
-  read (fd, &hdr, sizeof hdr);
-  if (*(Elf32_Word *) hdr.e.e_ident == *(Elf32_Word *) "\177ELF")
-    {
-      Elf32_Phdr phdrs[hdr.e.e_phnum], *ph;
-      lseek (fd, hdr.e.e_phoff, SEEK_SET);
-      read (fd, phdrs, sizeof phdrs);
-      for (ph = phdrs; ph < &phdrs[sizeof phdrs/sizeof phdrs[0]]; ++ph)
-	if (ph->p_type == PT_LOAD)
-	  {
-	    vm_address_t buf;
-	    vm_size_t offs = ph->p_offset & (ph->p_align - 1);
-	    vm_size_t bufsz = round_page (ph->p_filesz + offs);
-
-	    buf = (vm_address_t) mmap (0, bufsz,
-				       PROT_READ|PROT_WRITE, MAP_ANON, 0, 0);
-
-	    lseek (fd, ph->p_offset, SEEK_SET);
-	    read (fd, (void *)(buf + offs), ph->p_filesz);
-
-	    ph->p_memsz = ((ph->p_vaddr + ph->p_memsz + ph->p_align - 1)
-			   & ~(ph->p_align - 1));
-	    ph->p_vaddr &= ~(ph->p_align - 1);
-	    ph->p_memsz -= ph->p_vaddr;
-
-	    vm_allocate (t, (vm_address_t*)&ph->p_vaddr, ph->p_memsz, 0);
-	    vm_write (t, ph->p_vaddr, buf, bufsz);
-	    munmap ((caddr_t) buf, bufsz);
-	    vm_protect (t, ph->p_vaddr, ph->p_memsz, 0,
-			((ph->p_flags & PF_R) ? VM_PROT_READ : 0) |
-			((ph->p_flags & PF_W) ? VM_PROT_WRITE : 0) |
-			((ph->p_flags & PF_X) ? VM_PROT_EXECUTE : 0));
-	  }
-      return hdr.e.e_entry;
-    }
-  else
-    {
-      /* a.out */
-      int magic = N_MAGIC (hdr.a);
-      int headercruft;
-      vm_address_t base = 0x10000;
-      int rndamount, amount;
-      vm_address_t bsspagestart, bssstart;
-      char *buf;
-
-      headercruft = sizeof (struct exec) * (magic == ZMAGIC);
-
-      amount = headercruft + hdr.a.a_text + hdr.a.a_data;
-      rndamount = round_page (amount);
-      buf = mmap (0, rndamount, PROT_READ|PROT_WRITE, MAP_ANON, 0, 0);
-      lseek (fd, sizeof hdr.a - headercruft, SEEK_SET);
-      read (fd, buf, amount);
-      vm_allocate (t, &base, rndamount, 0);
-      vm_write (t, base, (vm_address_t) buf, rndamount);
-      if (magic != OMAGIC)
-	vm_protect (t, base, trunc_page (headercruft + hdr.a.a_text),
-		    0, VM_PROT_READ | VM_PROT_EXECUTE);
-      munmap ((caddr_t) buf, rndamount);
-
-      bssstart = base + hdr.a.a_text + hdr.a.a_data + headercruft;
-      bsspagestart = round_page (bssstart);
-      vm_allocate (t, &bsspagestart,
-		   hdr.a.a_bss - (bsspagestart - bssstart), 0);
-
-      return hdr.a.a_entry;
-    }
-}
-
-
 void read_reply ();
 void * msg_thread (void *);
 
-/* Callbacks for boot_script.c; see boot_script.h.  */
-
-mach_port_t
-boot_script_read_file (const char *filename)
-{
-  static const char msg[] = ": cannot open\n";
-  int fd = useropen (filename, O_RDONLY, 0);
-  host_stat_t st;
-  error_t err;
-  mach_port_t memobj;
-  vm_address_t region;
-
-  write (2, filename, strlen (filename));
-  if (fd < 0)
-    {
-      write (2, msg, sizeof msg - 1);
-      host_exit (1);
-    }
-  else
-    write (2, msg + sizeof msg - 2, 1);
-
-  host_fstat (fd, &st);
-
-  err = default_pager_object_create (defpager, &memobj,
-				     round_page (st.st_size));
-  if (err)
-    {
-      static const char msg[] = "cannot create default-pager object\n";
-      write (2, msg, sizeof msg - 1);
-      host_exit (1);
-    }
-
-  region = 0;
-  vm_map (mach_task_self (), &region, round_page (st.st_size),
-	  0, 1, memobj, 0, 0, VM_PROT_ALL, VM_PROT_ALL, VM_INHERIT_NONE);
-  read (fd, (char *) region, st.st_size);
-  munmap ((caddr_t) region, round_page (st.st_size));
-
-  close (fd);
-  return memobj;
-}
-
-int
-boot_script_exec_cmd (void *hook,
-		      mach_port_t task, char *path, int argc,
-		      char **argv, char *strings, int stringlen)
-{
-  char *args, *p;
-  int arg_len, i;
-  size_t reg_size;
-  void *arg_pos;
-  vm_offset_t stack_start, stack_end;
-  vm_address_t startpc, str_start;
-  thread_t thread;
-
-  write (2, path, strlen (path));
-  for (i = 1; i < argc; ++i)
-    {
-      write (2, " ", 1);
-      write (2, argv[i], strlen (argv[i]));
-    }
-  write (2, "\r\n", 2);
-
-  startpc = load_image (task, path);
-  arg_len = stringlen + (argc + 2) * sizeof (char *) + sizeof (integer_t);
-  arg_len += 5 * sizeof (int);
-  stack_end = VM_MAX_ADDRESS;
-  stack_start = VM_MAX_ADDRESS - 16 * 1024 * 1024;
-  vm_allocate (task, &stack_start, stack_end - stack_start, FALSE);
-  arg_pos = (void *) ((stack_end - arg_len) & ~(sizeof (natural_t) - 1));
-  args = mmap (0, stack_end - trunc_page ((vm_offset_t) arg_pos),
-	       PROT_READ|PROT_WRITE, MAP_ANON, 0, 0);
-  str_start = ((vm_address_t) arg_pos
-	       + (argc + 2) * sizeof (char *) + sizeof (integer_t));
-  p = args + ((vm_address_t) arg_pos & (vm_page_size - 1));
-  *(int *) p = argc;
-  p = (void *) p + sizeof (int);
-  for (i = 0; i < argc; i++)
-    {
-      *(char **) p = argv[i] - strings + (char *) str_start;
-      p = (void *) p + sizeof (char *);
-    }
-  *(char **) p = 0;
-  p = (void *) p + sizeof (char *);
-  *(char **) p = 0;
-  p = (void *) p + sizeof (char *);
-  memcpy (p, strings, stringlen);
-  bzero (args, (vm_offset_t) arg_pos & (vm_page_size - 1));
-  vm_write (task, trunc_page ((vm_offset_t) arg_pos), (vm_address_t) args,
-	    stack_end - trunc_page ((vm_offset_t) arg_pos));
-  munmap ((caddr_t) args,
-	  stack_end - trunc_page ((vm_offset_t) arg_pos));
-
-  thread_create (task, &thread);
-#ifdef i386_THREAD_STATE_COUNT
-  {
-    struct i386_thread_state regs;
-    reg_size = i386_THREAD_STATE_COUNT;
-    thread_get_state (thread, i386_THREAD_STATE,
-		      (thread_state_t) &regs, &reg_size);
-    regs.eip = (int) startpc;
-    regs.uesp = (int) arg_pos;
-    thread_set_state (thread, i386_THREAD_STATE,
-		      (thread_state_t) &regs, reg_size);
-  }
-#elif defined(ALPHA_THREAD_STATE_COUNT)
-  {
-    struct alpha_thread_state regs;
-    reg_size = ALPHA_THREAD_STATE_COUNT;
-    thread_get_state (thread, ALPHA_THREAD_STATE,
-		      (thread_state_t) &regs, &reg_size);
-    regs.r30 = (natural_t) arg_pos;
-    regs.pc = (natural_t) startpc;
-    thread_set_state (thread, ALPHA_THREAD_STATE,
-		      (thread_state_t) &regs, reg_size);
-  }
-#else
-# error needs to be ported
-#endif
-
-  thread_resume (thread);
-  mach_port_deallocate (mach_task_self (), thread);
-  return 0;
-}
-
 const char *argp_program_version = STANDARD_HURD_VERSION (boot);
 
+#define OPT_PRIVILEGED	-1
+#define OPT_BOOT_SCRIPT	-2
+
 static struct argp_option options[] =
 {
+  { NULL, 0, NULL, 0, "Boot options:" },
+  { "boot-script", OPT_BOOT_SCRIPT, "BOOT-SCRIPT", 0,
+    "boot script to execute" },
   { "boot-root",   'D', "DIR", 0,
     "Root of a directory tree in which to find files specified in BOOT-SCRIPT" },
   { "single-user", 's', 0, 0,
@@ -432,32 +233,46 @@ static struct argp_option options[] =
   { "pause" ,      'd', 0, 0,
     "Pause for user confirmation at various times during booting" },
   { "isig",      'I', 0, 0,
-    "Do not disable terminal signals, so you can suspend and interrupt boot."},
-  { "device",	   'f', "device_name=device_file", 0,
-    "Specify a device file used by subhurd and its virtual name."},
+    "Do not disable terminal signals, so you can suspend and interrupt boot"},
+  { "device",	   'f', "SUBHURD_NAME=DEVICE_FILE", 0,
+    "Pass the given DEVICE_FILE to the Subhurd as device SUBHURD_NAME"},
+  { "privileged", OPT_PRIVILEGED, NULL, 0,
+    "Allow the subhurd to access privileged kernel ports"},
   { 0 }
 };
-static char args_doc[] = "BOOT-SCRIPT";
 static char doc[] = "Boot a second hurd";
 
-struct dev_map 
+
+
+/* Device pass through.  */
+
+struct dev_map
 {
-  char *name;
-  mach_port_t port;
+  char *device_name;	/* The name of the device in the Subhurd.  */
+  char *file_name;	/* The filename outside the Subhurd.  */
   struct dev_map *next;
 };
 
 static struct dev_map *dev_map_head;
 
-static struct dev_map *add_dev_map (char *dev_name, char *dev_file)
+static struct dev_map *
+add_dev_map (const char *dev_name, const char *dev_file)
 {
-  struct dev_map *map = malloc (sizeof (*map));
+  file_t node;
+  struct dev_map *map;
 
-  assert (map);
-  map->name = dev_name;
-  map->port = file_name_lookup (dev_file, 0, 0);
-  if (map->port == MACH_PORT_NULL)
-    error (1, errno, "file_name_lookup: %s", dev_file);
+  /* See if we can open the file.  */
+  node = file_name_lookup (dev_file, 0, 0);
+  if (! MACH_PORT_VALID (node))
+    error (1, errno, "%s", dev_file);
+  mach_port_deallocate (mach_task_self (), node);
+
+  map = malloc (sizeof *map);
+  if (map == NULL)
+    return NULL;
+
+  map->device_name = strdup (dev_name);
+  map->file_name = strdup (dev_file);
   map->next = dev_map_head;
   dev_map_head = map;
   return map;
@@ -469,7 +284,7 @@ static struct dev_map *lookup_dev (char *dev_name)
 
   for (map = dev_map_head; map; map = map->next)
     {
-      if (strcmp (map->name, dev_name) == 0)
+      if (strcmp (map->device_name, dev_name) == 0)
 	return map;
     }
   return NULL;
@@ -506,13 +321,17 @@ parse_opt (int key, char *arg, struct argp_state *state)
       add_dev_map (arg, dev_file+1);
       break;
 
-    case ARGP_KEY_ARG:
-      if (state->arg_num == 0)
-	bootscript = arg;
-      else
-	return ARGP_ERR_UNKNOWN;
+    case OPT_PRIVILEGED:
+      want_privileged = 1;
       break;
 
+    case OPT_BOOT_SCRIPT:
+      bootscript = arg;
+      break;
+
+    case ARGP_KEY_ARG:
+      return ARGP_ERR_UNKNOWN;
+
     case ARGP_KEY_INIT:
       state->child_inputs[0] = state->input; break;
 
@@ -522,17 +341,142 @@ parse_opt (int key, char *arg, struct argp_state *state)
   return 0;
 }
 
+static error_t
+allocate_pseudo_ports (void)
+{
+  mach_port_t old;
+
+  /* Allocate a port that we hand out as the privileged host port.  */
+  mach_port_allocate (mach_task_self (), MACH_PORT_RIGHT_RECEIVE,
+		      &pseudo_privileged_host_port);
+  mach_port_insert_right (mach_task_self (),
+			  pseudo_privileged_host_port,
+			  pseudo_privileged_host_port,
+			  MACH_MSG_TYPE_MAKE_SEND);
+  mach_port_move_member (mach_task_self (), pseudo_privileged_host_port,
+			 receive_set);
+  mach_port_request_notification (mach_task_self (),
+                                  pseudo_privileged_host_port,
+				  MACH_NOTIFY_NO_SENDERS, 1,
+				  pseudo_privileged_host_port,
+				  MACH_MSG_TYPE_MAKE_SEND_ONCE, &old);
+  assert (old == MACH_PORT_NULL);
+
+  /* Allocate a port that we hand out as the privileged processor set
+     port.  */
+  mach_port_allocate (mach_task_self (), MACH_PORT_RIGHT_RECEIVE,
+		      &pseudo_pset);
+  mach_port_move_member (mach_task_self (), pseudo_pset,
+			 receive_set);
+  /* Make one send right that we copy when handing it out.  */
+  mach_port_insert_right (mach_task_self (),
+			  pseudo_pset,
+			  pseudo_pset,
+			  MACH_MSG_TYPE_MAKE_SEND);
+
+  /* We will receive new task notifications on this port.  */
+  mach_port_allocate (mach_task_self (), MACH_PORT_RIGHT_RECEIVE,
+		      &task_notification_port);
+  mach_port_move_member (mach_task_self (), task_notification_port,
+			 receive_set);
+
+  /* And information about dying tasks here.  */
+  mach_port_allocate (mach_task_self (), MACH_PORT_RIGHT_RECEIVE,
+		      &dead_task_notification_port);
+  mach_port_move_member (mach_task_self (), dead_task_notification_port,
+			 receive_set);
+
+  return 0;
+}
+
+void
+read_boot_script (char **buffer, size_t *length)
+{
+  char *p, *buf;
+  static const char filemsg[] = "Can't open boot script\n";
+  static const char memmsg[] = "Not enough memory\n";
+  int i, fd;
+  size_t amt, len;
+
+  fd = open (bootscript, O_RDONLY, 0);
+  if (fd < 0)
+    {
+      write (2, filemsg, sizeof (filemsg));
+      host_exit (1);
+    }
+  p = buf = malloc (500);
+  if (!buf)
+    {
+      write (2, memmsg, sizeof (memmsg));
+      host_exit (1);
+    }
+  len = 500;
+  amt = 0;
+  while (1)
+    {
+      i = read (fd, p, len - (p - buf));
+      if (i <= 0)
+        break;
+      p += i;
+      amt += i;
+      if (p == buf + len)
+        {
+          char *newbuf;
+
+          len += 500;
+          newbuf = realloc (buf, len);
+          if (!newbuf)
+            {
+              write (2, memmsg, sizeof (memmsg));
+              host_exit (1);
+            }
+          p = newbuf + (p - buf);
+          buf = newbuf;
+        }
+    }
+
+  close (fd);
+  *buffer = buf;
+  *length = amt;
+}
+
+
+/* Boot script file for booting contemporary GNU Hurd systems.  Each
+   line specifies a file to be loaded by the boot loader (the first
+   word), and actions to be done with it.  */
+const char *default_boot_script =
+  /* First, the bootstrap filesystem.  It needs several ports as
+     arguments, as well as the user flags from the boot loader.  */
+  "/hurd/ext2fs.static"
+  " --readonly"
+  " --multiboot-command-line=${kernel-command-line}"
+  " --host-priv-port=${host-port}"
+  " --device-master-port=${device-port}"
+  " --exec-server-task=${exec-task}"
+  " -T device ${root-device} $(task-create) $(task-resume)"
+  "\n"
+
+  /* Now the exec server; to load the dynamically-linked exec server
+     program, we have the boot loader in fact load and run ld.so,
+     which in turn loads and runs /hurd/exec.  This task is created,
+     and its task port saved in ${exec-task} to be passed to the fs
+     above, but it is left suspended; the fs will resume the exec task
+     once it is ready.  */
+  "/lib/ld.so /hurd/exec $(exec-task=task-create)"
+  "\n";
+
+
 int
 main (int argc, char **argv, char **envp)
 {
   error_t err;
   mach_port_t foo;
   char *buf = 0;
-  int i, len;
   pthread_t pthread_id;
   char *root_store_name;
-  const struct argp_child kids[] = { { &store_argp }, { 0 }};
-  struct argp argp = { options, parse_opt, args_doc, doc, kids };
+  const struct argp_child kids[] = { { &store_argp, 0, "Store options:", -2 },
+                                     { 0 }};
+  struct argp argp = { options, parse_opt, NULL, doc, kids };
   struct store_argp_params store_argp_params = { 0 };
 
   argp_parse (&argp, argc, argv, 0, 0, &store_argp_params);
@@ -544,19 +488,26 @@ main (int argc, char **argv, char **envp)
   if (err)
     error (4, err, "%s", root_store_name);
 
-  get_privileged_ports (&privileged_host_port, &master_device_port);
+  if (want_privileged)
+    {
+      get_privileged_ports (&privileged_host_port, &master_device_port);
+      privileged = MACH_PORT_VALID (master_device_port);
 
-  defpager = MACH_PORT_NULL;
-  vm_set_default_memory_manager (privileged_host_port, &defpager);
+      if (! privileged)
+        error (1, 0, "Must be run as root for privileged subhurds");
+    }
 
-  strcat (bootstrap_args, "f");
+  if (privileged)
+    strcat (bootstrap_args, "f");
 
   mach_port_allocate (mach_task_self (), MACH_PORT_RIGHT_PORT_SET,
 		      &receive_set);
 
   if (root_store->class == &store_device_class && root_store->name
       && (root_store->flags & STORE_ENFORCED)
-      && root_store->num_runs == 1 && root_store->runs[0].start == 0)
+      && root_store->num_runs == 1
+      && root_store->runs[0].start == 0
+      && privileged)
     /* Let known device nodes pass through directly.  */
     bootdevice = root_store->name;
   else
@@ -592,13 +543,42 @@ main (int argc, char **argv, char **envp)
   if (foo != MACH_PORT_NULL)
     mach_port_deallocate (mach_task_self (), foo);
 
+  mach_port_allocate (mach_task_self (), MACH_PORT_RIGHT_RECEIVE,
+		      &pseudo_time);
+  mach_port_move_member (mach_task_self (), pseudo_time, receive_set);
+  mach_port_request_notification (mach_task_self (), pseudo_time,
+				  MACH_NOTIFY_NO_SENDERS, 1, pseudo_time,
+				  MACH_MSG_TYPE_MAKE_SEND_ONCE, &foo);
+  if (foo != MACH_PORT_NULL)
+    mach_port_deallocate (mach_task_self (), foo);
+
+  if (! privileged)
+    {
+      err = allocate_pseudo_ports ();
+      if (err)
+        error (1, err, "Allocating pseudo ports");
+
+      /* Create a new task namespace for us.  */
+      err = proc_make_task_namespace (getproc (), task_notification_port,
+                                      MACH_MSG_TYPE_MAKE_SEND);
+      if (err)
+        error (1, err, "proc_make_task_namespace");
+
+      /* Create an empty task that the subhurds can freely frobnicate.  */
+      err = task_create (mach_task_self (), 0, &pseudo_kernel);
+      if (err)
+        error (1, err, "task_create");
+    }
+
   if (kernel_command_line == 0)
     asprintf (&kernel_command_line, "%s %s root=%s",
 	      argv[0], bootstrap_args, bootdevice);
 
   /* Initialize boot script variables.  */
   if (boot_script_set_variable ("host-port", VAL_PORT,
-				(int) privileged_host_port)
+                                privileged
+                                ? (int) privileged_host_port
+				: (int) pseudo_privileged_host_port)
       || boot_script_set_variable ("device-port", VAL_PORT,
 				   (integer_t) pseudo_master_device_port)
       || boot_script_set_variable ("kernel-command-line", VAL_STR,
@@ -644,46 +624,12 @@ main (int argc, char **argv, char **envp)
   /* Parse the boot script.  */
   {
     char *p, *line;
-    static const char filemsg[] = "Can't open boot script\n";
-    static const char memmsg[] = "Not enough memory\n";
-    int amt, fd, err;
+    size_t amt;
+    if (bootscript)
+      read_boot_script (&buf, &amt);
+    else
+      buf = strdup (default_boot_script), amt = strlen (default_boot_script);
 
-    fd = open (bootscript, O_RDONLY, 0);
-    if (fd < 0)
-      {
-	write (2, filemsg, sizeof (filemsg));
-	host_exit (1);
-      }
-    p = buf = malloc (500);
-    if (!buf)
-      {
-	write (2, memmsg, sizeof (memmsg));
-	host_exit (1);
-      }
-    len = 500;
-    amt = 0;
-    while (1)
-      {
-	i = read (fd, p, len - (p - buf));
-	if (i <= 0)
-	  break;
-	p += i;
-	amt += i;
-	if (p == buf + len)
-	  {
-	    char *newbuf;
-
-	    len += 500;
-	    newbuf = realloc (buf, len);
-	    if (!newbuf)
-	      {
-		write (2, memmsg, sizeof (memmsg));
-		host_exit (1);
-	      }
-	    p = newbuf + (p - buf);
-	    buf = newbuf;
-	  }
-      }
     line = p = buf;
     while (1)
       {
@@ -723,8 +669,6 @@ main (int argc, char **argv, char **envp)
   /* The boot script has now been parsed into internal data structures.
      Now execute its directives.  */
   {
-    int err;
-
     err = boot_script_exec ();
     if (err)
       {
@@ -756,18 +700,17 @@ main (int argc, char **argv, char **envp)
       FD_SET (0, &rmask);
       if (select (1, &rmask, 0, 0, 0) == 1)
 	read_reply ();
-      else /* We hosed */
+      else if (errno != EINTR)
+        /* We hosed */
 	error (5, errno, "select");
     }
-
-/*  mach_msg_server (request_server, __vm_page_size * 2, receive_set); */
 }
 
 void *
 msg_thread (void *arg)
 {
   while (1)
-    mach_msg_server (request_server, 0, receive_set);
+    mach_msg_server (boot_demuxer, 0, receive_set);
 }
 
 
@@ -914,8 +857,6 @@ unlock_readlock ()
 /*
  *	Handle bootstrap requests.
  */
-/* These two functions from .../mk/bootstrap/default_pager.c. */
-
 kern_return_t
 do_bootstrap_privileged_ports(bootstrap, hostp, devicep)
 	mach_port_t bootstrap;
@@ -925,68 +866,6 @@ do_bootstrap_privileged_ports(bootstrap, hostp, devicep)
 	*devicep = pseudo_master_device_port;
 	return KERN_SUCCESS;
 }
-
-void
-bootstrap_compat(in, out)
-	mach_msg_header_t *in, *out;
-{
-	mig_reply_header_t *reply = (mig_reply_header_t *) out;
-	mach_msg_return_t mr;
-
-	struct imsg {
-		mach_msg_header_t	hdr;
-		mach_msg_type_t		port_desc_1;
-		mach_port_t		port_1;
-		mach_msg_type_t		port_desc_2;
-		mach_port_t		port_2;
-	} imsg;
-
-	/*
-	 * Send back the host and device ports.
-	 */
-
-	imsg.hdr.msgh_bits = MACH_MSGH_BITS_COMPLEX |
-		MACH_MSGH_BITS(MACH_MSGH_BITS_REMOTE(in->msgh_bits), 0);
-	/* msgh_size doesn't need to be initialized */
-	imsg.hdr.msgh_remote_port = in->msgh_remote_port;
-	imsg.hdr.msgh_local_port = MACH_PORT_NULL;
-	/* msgh_seqno doesn't need to be initialized */
-	imsg.hdr.msgh_id = in->msgh_id + 100;	/* this is a reply msg */
-
-	imsg.port_desc_1.msgt_name = MACH_MSG_TYPE_COPY_SEND;
-	imsg.port_desc_1.msgt_size = (sizeof(mach_port_t) * 8);
-	imsg.port_desc_1.msgt_number = 1;
-	imsg.port_desc_1.msgt_inline = TRUE;
-	imsg.port_desc_1.msgt_longform = FALSE;
-	imsg.port_desc_1.msgt_deallocate = FALSE;
-	imsg.port_desc_1.msgt_unused = 0;
-
-	imsg.port_1 = privileged_host_port;
-
-	imsg.port_desc_2 = imsg.port_desc_1;
-
-	imsg.port_desc_2.msgt_name = MACH_MSG_TYPE_MAKE_SEND;
-	imsg.port_2 = pseudo_master_device_port;
-
-	/*
-	 * Send the reply message.
-	 * (mach_msg_server can not do this, because the reply
-	 * is not in standard format.)
-	 */
-
-	mr = mach_msg(&imsg.hdr, MACH_SEND_MSG,
-		      sizeof imsg, 0, MACH_PORT_NULL,
-		      MACH_MSG_TIMEOUT_NONE, MACH_PORT_NULL);
-	if (mr != MACH_MSG_SUCCESS)
-		(void) mach_port_deallocate(mach_task_self (),
-					    imsg.hdr.msgh_remote_port);
-
-	/*
-	 * Tell mach_msg_server to do nothing.
-	 */
-
-	reply->RetCode = MIG_NO_REPLY;
-}
 
 /* Implementation of device interface */
 
@@ -1016,6 +895,12 @@ ds_device_open (mach_port_t master_port,
       *devicetype = MACH_MSG_TYPE_MAKE_SEND;
       return 0;
     }
+  else if (!strcmp (name, "time"))
+    {
+      *device = pseudo_time;
+      *devicetype = MACH_MSG_TYPE_MAKE_SEND;
+      return 0;
+    }
   else if (strcmp (name, "pseudo-root") == 0)
     /* Magic root device.  */
     {
@@ -1027,8 +912,17 @@ ds_device_open (mach_port_t master_port,
   map = lookup_dev (name);
   if (map)
     {
+      error_t err;
+      file_t node;
+
+      node = file_name_lookup (map->file_name, 0, 0);
+      if (! MACH_PORT_VALID (node))
+        return D_NO_SUCH_DEVICE;
+
       *devicetype = MACH_MSG_TYPE_MOVE_SEND;
-      return device_open (map->port, mode, "", device);
+      err = device_open (node, mode, "", device);
+      mach_port_deallocate (mach_task_self (), node);
+      return err;
     }
 
   *devicetype = MACH_MSG_TYPE_MOVE_SEND;
@@ -1239,40 +1133,6 @@ ds_device_read_inband (device_t device,
     return D_NO_SUCH_DEVICE;
 }
 
-kern_return_t
-ds_xxx_device_set_status (device_t device,
-			  dev_flavor_t flavor,
-			  dev_status_t status,
-			  size_t statu_cnt)
-{
-  if (device != pseudo_console)
-    return D_NO_SUCH_DEVICE;
-  return D_INVALID_OPERATION;
-}
-
-kern_return_t
-ds_xxx_device_get_status (device_t device,
-			  dev_flavor_t flavor,
-			  dev_status_t status,
-			  size_t *statuscnt)
-{
-  if (device != pseudo_console && device != pseudo_root)
-    return D_NO_SUCH_DEVICE;
-  return D_INVALID_OPERATION;
-}
-
-kern_return_t
-ds_xxx_device_set_filter (device_t device,
-			  mach_port_t rec,
-			  int pri,
-			  filter_array_t filt,
-			  size_t len)
-{
-  if (device != pseudo_console && device != pseudo_root)
-    return D_NO_SUCH_DEVICE;
-  return D_INVALID_OPERATION;
-}
-
 kern_return_t
 ds_device_map (device_t device,
 	       vm_prot_t prot,
@@ -1281,9 +1141,26 @@ ds_device_map (device_t device,
 	       memory_object_t *pager,
 	       int unmap)
 {
-  if (device != pseudo_console && device != pseudo_root)
+  if (device == pseudo_console || device == pseudo_root)
+    return D_INVALID_OPERATION;
+  else if (device == pseudo_time)
+    {
+      error_t err;
+      mach_port_t wr_memobj;
+      file_t node = file_name_lookup ("/dev/time", O_RDONLY, 0);
+
+      if (node == MACH_PORT_NULL)
+	return D_IO_ERROR;
+
+      err = io_map (node, pager, &wr_memobj);
+      if (!err && MACH_PORT_VALID (wr_memobj))
+	mach_port_deallocate (mach_task_self (), wr_memobj);
+
+      mach_port_deallocate (mach_task_self (), node);
+      return D_SUCCESS;
+    }
+  else
     return D_NO_SUCH_DEVICE;
-  return D_INVALID_OPERATION;
 }
 
 kern_return_t
@@ -1306,18 +1183,27 @@ ds_device_get_status (device_t device,
   if (device == pseudo_console)
     return D_INVALID_OPERATION;
   else if (device == pseudo_root)
-    if (flavor == DEV_GET_SIZE)
-      if (*statuslen < DEV_GET_SIZE_COUNT)
-	return D_INVALID_SIZE;
-      else
-	{
-	  status[DEV_GET_SIZE_DEVICE_SIZE] = root_store->size;
-	  status[DEV_GET_SIZE_RECORD_SIZE] = root_store->block_size;
-	  *statuslen = DEV_GET_SIZE_COUNT;
-	  return D_SUCCESS;
-	}
-    else
-      return D_INVALID_OPERATION;
+    switch (flavor)
+      {
+      case DEV_GET_SIZE:
+        if (*statuslen < DEV_GET_SIZE_COUNT)
+          return D_INVALID_SIZE;
+        status[DEV_GET_SIZE_DEVICE_SIZE] = root_store->size;
+        status[DEV_GET_SIZE_RECORD_SIZE] = root_store->block_size;
+        *statuslen = DEV_GET_SIZE_COUNT;
+        return D_SUCCESS;
+
+      case DEV_GET_RECORDS:
+        if (*statuslen < DEV_GET_RECORDS_COUNT)
+          return D_INVALID_SIZE;
+        status[DEV_GET_RECORDS_DEVICE_RECORDS] = root_store->blocks;
+        status[DEV_GET_RECORDS_RECORD_SIZE] = root_store->block_size;
+        *statuslen = DEV_GET_RECORDS_COUNT;
+        return D_SUCCESS;
+
+      default:
+        return D_INVALID_OPERATION;
+      }
   else
     return D_NO_SUCH_DEVICE;
 }
@@ -1405,6 +1291,8 @@ do_mach_notify_send_once (mach_port_t notify)
   return EOPNOTSUPP;
 }
 
+static void task_died (mach_port_t name);
+
 kern_return_t
 do_mach_notify_dead_name (mach_port_t notify,
 			  mach_port_t name)
@@ -1413,7 +1301,11 @@ do_mach_notify_dead_name (mach_port_t notify,
   if (name == child_task && notify == bootport)
     host_exit (0);
 #endif
-  return EOPNOTSUPP;
+  if (notify != dead_task_notification_port)
+    return EOPNOTSUPP;
+  task_died (name);
+  mach_port_deallocate (mach_task_self (), name);
+  return 0;
 }
 
 
@@ -1663,7 +1555,7 @@ S_io_stat (mach_port_t object,
   if (object != pseudo_console)
     return EOPNOTSUPP;
 
-  bzero (st, sizeof (struct stat));
+  memset (st, 0, sizeof(struct stat));
   st->st_blksize = 1024;
   return 0;
 }
@@ -1679,17 +1571,23 @@ S_io_reauthenticate (mach_port_t object,
   size_t gulen = 0, aulen = 0, gglen = 0, aglen = 0;
   error_t err;
 
+  /* XXX: This cannot possibly work, authserver is 0.  */
+
   err = mach_port_insert_right (mach_task_self (), object, object,
 				MACH_MSG_TYPE_MAKE_SEND);
   assert_perror (err);
 
-  if (! auth_server_authenticate (authserver,
+  do
+    err = auth_server_authenticate (authserver,
 				  rend, MACH_MSG_TYPE_COPY_SEND,
 				  object, MACH_MSG_TYPE_COPY_SEND,
 				  &gu, &gulen,
 				  &au, &aulen,
 				  &gg, &gglen,
-				  &ag, &aglen))
+				  &ag, &aglen);
+  while (err == EINTR);
+
+  if (!err)
     {
       mig_deallocate ((vm_address_t) gu, gulen * sizeof *gu);
       mig_deallocate ((vm_address_t) au, aulen * sizeof *gu);
@@ -1876,7 +1774,7 @@ S_io_revoke (mach_port_t obj,
    support on the console device.  */
 
 kern_return_t
-S_termctty_open_terminal (mach_port_t object,
+S_termctty_open_terminal (ctty_t object,
 			  int flags,
 			  mach_port_t *result,
 			  mach_msg_type_name_t *restype)
@@ -1967,3 +1865,160 @@ kern_return_t S_term_on_pty
 	io_t *ptymaster
 )
 { return EOPNOTSUPP; }
+
+/* Mach host emulation.  */
+
+kern_return_t
+S_vm_set_default_memory_manager (mach_port_t host_priv,
+                                 mach_port_t *default_manager)
+{
+  if (host_priv != pseudo_privileged_host_port)
+    return KERN_INVALID_HOST;
+
+  if (*default_manager != MACH_PORT_NULL)
+    return KERN_INVALID_ARGUMENT;
+
+  *default_manager = MACH_PORT_NULL;
+  return KERN_SUCCESS;
+}
+
+kern_return_t
+S_host_reboot (mach_port_t host_priv,
+               int flags)
+{
+  fprintf (stderr, "Would %s the system.  Bye.\n",
+           flags & RB_HALT? "halt": "reboot");
+  host_exit (0);
+}
+
+
+kern_return_t
+S_host_processor_set_priv (mach_port_t host_priv,
+			   mach_port_t set_name,
+			   mach_port_t *set)
+{
+  if (host_priv != pseudo_privileged_host_port)
+    return KERN_INVALID_HOST;
+
+  *set = pseudo_pset;
+  return KERN_SUCCESS;
+}
+
+mach_port_t new_task_notification;
+
+kern_return_t
+S_register_new_task_notification (mach_port_t host_priv,
+				  mach_port_t notification)
+{
+  if (host_priv != pseudo_privileged_host_port)
+    return KERN_INVALID_HOST;
+
+  if (! MACH_PORT_VALID (notification))
+    return KERN_INVALID_ARGUMENT;
+
+  if (MACH_PORT_VALID (new_task_notification))
+    return KERN_NO_ACCESS;
+
+  new_task_notification = notification;
+  return KERN_SUCCESS;
+}
+
+
+/* Managing tasks.  */
+
+static void
+task_ihash_cleanup (hurd_ihash_value_t value, void *cookie)
+{
+  (void) cookie;
+  mach_port_deallocate (mach_task_self (), (mach_port_t) value);
+}
+
+static struct hurd_ihash task_ihash =
+  HURD_IHASH_INITIALIZER_GKI (HURD_IHASH_NO_LOCP, task_ihash_cleanup, NULL,
+                              NULL, NULL);
+
+static void
+task_died (mach_port_t name)
+{
+  hurd_ihash_remove (&task_ihash, (hurd_ihash_key_t) name);
+}
+
+/* Handle new task notifications from proc.  */
+error_t
+S_mach_notify_new_task (mach_port_t notify,
+			mach_port_t task,
+			mach_port_t parent)
+{
+  error_t err;
+  mach_port_t previous;
+
+  if (notify != task_notification_port)
+    return EOPNOTSUPP;
+
+  err = mach_port_request_notification (mach_task_self (), task,
+                                        MACH_NOTIFY_DEAD_NAME, 0,
+                                        dead_task_notification_port,
+                                        MACH_MSG_TYPE_MAKE_SEND_ONCE,
+                                        &previous);
+  if (err)
+    goto fail;
+  assert (! MACH_PORT_VALID (previous));
+
+  mach_port_mod_refs (mach_task_self (), task, MACH_PORT_RIGHT_SEND, +1);
+  err = hurd_ihash_add (&task_ihash,
+                        (hurd_ihash_key_t) task, (hurd_ihash_value_t) task);
+  if (err)
+    {
+      mach_port_deallocate (mach_task_self (), task);
+      goto fail;
+    }
+
+  if (MACH_PORT_VALID (new_task_notification))
+    /* Relay the notification.  This consumes task and parent.  */
+    return mach_notify_new_task (new_task_notification, task, parent);
+
+  mach_port_deallocate (mach_task_self (), task);
+  mach_port_deallocate (mach_task_self (), parent);
+  return 0;
+
+ fail:
+  task_terminate (task);
+  return err;
+}
+
+kern_return_t
+S_processor_set_tasks(mach_port_t processor_set,
+		      task_array_t *task_list,
+		      mach_msg_type_number_t *task_listCnt)
+{
+  error_t err;
+  size_t i;
+
+  if (!task_ihash.nr_items)
+    {
+      *task_listCnt = 0;
+      return 0;
+    }
+
+  err = vm_allocate (mach_task_self (), (vm_address_t *) task_list,
+		     task_ihash.nr_items * sizeof **task_list, 1);
+  if (err)
+    return err;
+
+  /* The first task has to be the kernel.  */
+  (*task_list)[0] = pseudo_kernel;
+
+  i = 1;
+  HURD_IHASH_ITERATE (&task_ihash, value)
+    {
+      task_t task = (task_t) value;
+      if (task == pseudo_kernel)
+        continue;
+
+      (*task_list)[i] = task;
+      i += 1;
+    }
+
+  *task_listCnt = task_ihash.nr_items;
+  return 0;
+}
