@@ -640,7 +640,7 @@ pager_unlock_page (struct user_pager_info *pager, vm_offset_t page)
       pthread_rwlock_unlock (&dn->alloc_lock);
 
       if (err == ENOSPC)
-	ext2_warning ("This filesystem is out of space, and will now crash.  Bye!");
+	ext2_warning ("This filesystem is out of space.");
       else if (err)
 	ext2_warning ("inode=%Ld, page=0x%lx: %s",
 		      node->cache_id, page, strerror (err));
@@ -817,22 +817,32 @@ pager_clear_user_data (struct user_pager_info *upi)
 
       pthread_spin_lock (&node_to_page_lock);
       pager = diskfs_node_disknode (upi->node)->pager;
-      if (pager && pager_get_upi (pager) == upi)
-	diskfs_node_disknode (upi->node)->pager = 0;
+      assert (!pager || pager_get_upi (pager) != upi);
       pthread_spin_unlock (&node_to_page_lock);
 
       diskfs_nrele_light (upi->node);
     }
-
-  free (upi);
 }
 
 /* This will be called when the ports library wants to drop weak references.
    The pager library creates no weak references itself.  If the user doesn't
    either, then it's OK for this function to do nothing.  */
 void
-pager_dropweak (struct user_pager_info *p __attribute__ ((unused)))
+pager_dropweak (struct user_pager_info *upi)
 {
+  if (upi->type == FILE_DATA)
+    {
+      struct pager *pager;
+
+      pthread_spin_lock (&node_to_page_lock);
+      pager = diskfs_node_disknode (upi->node)->pager;
+      if (pager && pager_get_upi (pager) == upi)
+	{
+	  diskfs_node_disknode (upi->node)->pager = NULL;
+	  ports_port_deref_weak (pager);
+	}
+      pthread_spin_unlock (&node_to_page_lock);
+    }
 }
 
 /* Cached blocks from disk.  */
@@ -1187,7 +1197,7 @@ disk_cache_block_ref_ptr (void *ptr)
 }
 
 void
-disk_cache_block_deref (void *ptr)
+_disk_cache_block_deref (void *ptr)
 {
   int index;
 
@@ -1298,34 +1308,30 @@ diskfs_get_filemap (struct node *node, vm_prot_t prot)
       struct pager *pager = diskfs_node_disknode (node)->pager;
       if (pager)
 	{
-	  /* Because PAGER is not a real reference,
-	     this might be nearly deallocated.  If that's so, then
-	     the port right will be null.  In that case, clear here
-	     and loop.  The deallocation will complete separately. */
 	  right = pager_get_port (pager);
-	  if (right == MACH_PORT_NULL)
-	    diskfs_node_disknode (node)->pager = 0;
-  	  else
-  	    pager_get_upi (pager)->max_prot |= prot;
+	  assert (MACH_PORT_VALID (right));
+	  pager_get_upi (pager)->max_prot |= prot;
 	}
       else
 	{
-	  struct user_pager_info *upi =
-	    malloc (sizeof (struct user_pager_info));
+	  struct user_pager_info *upi;
+	  pager = pager_create_alloc (sizeof *upi, file_pager_bucket,
+				      MAY_CACHE, MEMORY_OBJECT_COPY_DELAY, 0);
+	  if (pager == NULL)
+	    {
+	      pthread_spin_unlock (&node_to_page_lock);
+	      return MACH_PORT_NULL;
+	    }
+
+	  upi = pager_get_upi (pager);
 	  upi->type = FILE_DATA;
 	  upi->node = node;
 	  upi->max_prot = prot;
 	  diskfs_nref_light (node);
-	  diskfs_node_disknode (node)->pager =
-		    pager_create (upi, file_pager_bucket, MAY_CACHE,
-				  MEMORY_OBJECT_COPY_DELAY, 0);
-	  if (diskfs_node_disknode (node)->pager == 0)
-	    {
-	      diskfs_nrele_light (node);
-	      free (upi);
-	      pthread_spin_unlock (&node_to_page_lock);
-	      return MACH_PORT_NULL;
-	    }
+	  diskfs_node_disknode (node)->pager = pager;
+
+	  /* A weak reference for being part of the node.  */
+	  ports_port_ref_weak (diskfs_node_disknode (node)->pager);
 
 	  right = pager_get_port (diskfs_node_disknode (node)->pager);
 	  ports_port_deref (diskfs_node_disknode (node)->pager);
