@@ -1,5 +1,6 @@
 /* libdiskfs implementation of fs.defs: file_set_translator
-   Copyright (C) 1992, 1993, 1994, 1995, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1992,93,94,95,96,99,2001,02,13,14
+     Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -20,9 +21,6 @@
 #include <hurd/paths.h>
 #include <hurd/fsys.h>
 
-/* XXX - Temporary */
-#define makedev(maj,min) ((((maj)&0xFF)<<8)+((min)&0xFF))
-
 /* Implement file_set_translator as described in <hurd/fs.defs>. */
 kern_return_t
 diskfs_S_file_set_translator (struct protid *cred,
@@ -30,11 +28,11 @@ diskfs_S_file_set_translator (struct protid *cred,
 			      int active_flags,
 			      int killtrans_flags,
 			      char *passive,
-			      u_int passivelen,
+			      size_t passivelen,
 			      fsys_t active)
 {
   struct node *np;
-  error_t error;
+  error_t err;
   mach_port_t control = MACH_PORT_NULL;
 
   if (!cred)
@@ -46,39 +44,40 @@ diskfs_S_file_set_translator (struct protid *cred,
   if ((passive_flags & FS_TRANS_SET) && diskfs_check_readonly ())
     return EROFS;
 
-  if (passive && passive[passivelen - 1])
+  if (passivelen && passive[passivelen - 1])
     return EINVAL;
 
   np = cred->po->np;
 
-  mutex_lock (&np->lock);
+  pthread_mutex_lock (&np->lock);
 
-  error = fshelp_isowner (&np->dn_stat, cred->user);
-  if (error)
+  err = fshelp_isowner (&np->dn_stat, cred->user);
+  if (err)
     {
-      mutex_unlock (&np->lock);
-      return error;
+      pthread_mutex_unlock (&np->lock);
+      return err;
     }
 
-  if (active_flags & FS_TRANS_SET)
+  if ((active_flags & FS_TRANS_SET)
+      && ! (active_flags & FS_TRANS_ORPHAN))
     {
-      error = fshelp_fetch_control (&np->transbox, &control);
-      if (error)
+      err = fshelp_fetch_control (&np->transbox, &control);
+      if (err)
 	{
-	  mutex_unlock (&np->lock);
-	  return error;
+	  pthread_mutex_unlock (&np->lock);
+	  return err;
 	}
 
       if ((control != MACH_PORT_NULL) && ((active_flags & FS_TRANS_EXCL) == 0))
 	{
-	  mutex_unlock (&np->lock);
-	  error = fsys_goaway (control, killtrans_flags);
+	  pthread_mutex_unlock (&np->lock);
+	  err = fsys_goaway (control, killtrans_flags);
 	  mach_port_deallocate (mach_task_self (), control);
-	  if (error && (error != MIG_SERVER_DIED)
-	      && (error != MACH_SEND_INVALID_DEST))
-	    return error;
-	  error = 0;
-	  mutex_lock (&np->lock);
+	  if (err && (err != MIG_SERVER_DIED)
+	      && (err != MACH_SEND_INVALID_DEST))
+	    return err;
+	  err = 0;
+	  pthread_mutex_lock (&np->lock);
 	}
       else if (control != MACH_PORT_NULL)
 	mach_port_deallocate (mach_task_self (), control);
@@ -89,18 +88,18 @@ diskfs_S_file_set_translator (struct protid *cred,
       && (passive_flags & FS_TRANS_EXCL)
       && (np->dn_stat.st_mode & S_IPTRANS))
     {
-      mutex_unlock (&np->lock);
+      pthread_mutex_unlock (&np->lock);
       return EBUSY;
     }
 
   if (active_flags & FS_TRANS_SET)
     {
-      error = fshelp_set_active (&np->transbox, active,
+      err = fshelp_set_active (&np->transbox, active,
 				 active_flags & FS_TRANS_EXCL);
-      if (error)
+      if (err)
 	{
-	  mutex_unlock (&np->lock);
-	  return error;
+	  pthread_mutex_unlock (&np->lock);
+	  return err;
 	}
     }
 
@@ -131,7 +130,7 @@ diskfs_S_file_set_translator (struct protid *cred,
 		     changes, the links will be lost.  Perhaps it might be
 		     allowed for empty directories, but that's too much of a
 		     pain.  */
-		  mutex_unlock (&np->lock);
+		  pthread_mutex_unlock (&np->lock);
 		  return EISDIR;
 		}
 	      if (newmode == S_IFBLK || newmode == S_IFCHR)
@@ -145,7 +144,7 @@ diskfs_S_file_set_translator (struct protid *cred,
 		  assert (arg <= passive + passivelen);
 		  if (arg == passive + passivelen)
 		    {
-		      mutex_unlock (&np->lock);
+		      pthread_mutex_unlock (&np->lock);
 		      return EINVAL;
 		    }
 		  major = strtol (arg, 0, 0);
@@ -154,59 +153,76 @@ diskfs_S_file_set_translator (struct protid *cred,
 		  assert (arg < passive + passivelen);
 		  if (arg == passive + passivelen)
 		    {
-		      mutex_unlock (&np->lock);
+		      pthread_mutex_unlock (&np->lock);
 		      return EINVAL;
 		    }
 		  minor = strtol (arg, 0, 0);
 
-		  error = diskfs_validate_rdev_change (np, 
+		  err = diskfs_validate_rdev_change (np,
 						       makedev (major, minor));
-		  if (error)
+		  if (err)
 		    {
-		      mutex_unlock (&np->lock);
-		      return error;
+		      pthread_mutex_unlock (&np->lock);
+		      return err;
 		    }
 		  np->dn_stat.st_rdev = makedev (major, minor);
 		}
 
-	      diskfs_truncate (np, 0);
+	      err = diskfs_truncate (np, 0);
+	      if (err)
+		{
+		  pthread_mutex_unlock (&np->lock);
+		  return err;
+		}
+
+	      err = diskfs_set_translator (np, NULL, 0, cred);
+	      if (err)
+		{
+		  pthread_mutex_unlock (&np->lock);
+		  return err;
+		}
+
 	      if (newmode == S_IFLNK)
 		{
 		  char *arg = passive + strlen (passive) + 1;
 		  assert (arg <= passive + passivelen);
 		  if (arg == passive + passivelen)
 		    {
-		      mutex_unlock (&np->lock);
+		      pthread_mutex_unlock (&np->lock);
 		      return EINVAL;
 		    }
 
 		  if (diskfs_create_symlink_hook)
-		    error = (*diskfs_create_symlink_hook)(np, arg);
-		  if (!diskfs_create_symlink_hook || error == EINVAL)
+		    err = (*diskfs_create_symlink_hook)(np, arg);
+		  if (!diskfs_create_symlink_hook || err == EINVAL)
 		    /* Store the argument in the file as the
 		       target of the link */
-		    error = diskfs_node_rdwr (np, arg, 0, strlen (arg),
+		    err = diskfs_node_rdwr (np, arg, 0, strlen (arg),
 					      1, cred, 0);
-		  if (error)
+		  if (err)
 		    {
-		      mutex_unlock (&np->lock);
-		      return error;
+		      pthread_mutex_unlock (&np->lock);
+		      return err;
 		    }
 		}
 	      newmode = (np->dn_stat.st_mode & ~S_IFMT) | newmode;
-	      error = diskfs_validate_mode_change (np, newmode);
-	      if (!error)
+	      err = diskfs_validate_mode_change (np, newmode);
+	      if (!err)
 		{
 		  np->dn_stat.st_mode = newmode;
-		  diskfs_node_update (np, 1);
+		  diskfs_node_update (np, diskfs_synchronous);
 		}
-	      mutex_unlock (&np->lock);
-	      return error;
+	      pthread_mutex_unlock (&np->lock);
+	      return err;
 	    }
 	}
-      error = diskfs_set_translator (np, passive, passivelen, cred);
+      err = diskfs_set_translator (np, passive, passivelen, cred);
     }
 
-  mutex_unlock (&np->lock);
-  return error;
+  pthread_mutex_unlock (&np->lock);
+
+  if (! err && cred->po->path && active_flags & FS_TRANS_SET)
+    err = fshelp_set_active_translator (&cred->pi, cred->po->path, active);
+
+  return err;
 }
