@@ -17,9 +17,22 @@
 
 #include "priv.h"
 
-/* Node NP now has no more references; clean all state.  The
-   diskfs_node_refcnt_lock must be held, and will be released
-   upon return.  NP must be locked.  */
+/* Free the list of modification requests MR */
+static void
+free_modreqs (struct modreq *mr)
+{
+  struct modreq *tmp;
+  for (; mr; mr = tmp)
+    {
+      mach_port_deallocate (mach_task_self (), mr->port);
+      tmp = mr->next;
+      free (mr);
+    }
+}
+
+
+/* Node NP now has no more references; clean all state.  NP must be
+   locked.  */
 void
 diskfs_drop_node (struct node *np)
 {
@@ -46,8 +59,7 @@ diskfs_drop_node (struct node *np)
 	     and an nput.  The next time through, this routine
 	     will notice that the size is zero, and not have to
 	     do anything. */
-	  np->references++;
-	  spin_unlock (&diskfs_node_refcnt_lock);
+	  refcounts_unsafe_ref (&np->refcounts, NULL);
 	  diskfs_truncate (np, 0);
 	  
 	  /* Force allocsize to zero; if truncate consistently fails this
@@ -64,7 +76,7 @@ diskfs_drop_node (struct node *np)
       np->dn_stat.st_mode = 0;
       np->dn_stat.st_rdev = 0;
       np->dn_set_ctime = np->dn_set_atime = 1;
-      diskfs_node_update (np, 1);
+      diskfs_node_update (np, diskfs_synchronous);
       diskfs_free_node (np, savemode);
     }
   else
@@ -73,20 +85,13 @@ diskfs_drop_node (struct node *np)
   fshelp_drop_transbox (&np->transbox);
 
   if (np->dirmod_reqs)
-    {
-      struct dirmod *dm, *tmp;
-      for (dm = np->dirmod_reqs; dm; dm = tmp)
-	{
-	  mach_port_deallocate (mach_task_self (), dm->port);
-	  tmp = dm->next;
-	  free (dm);
-	}
-    }
+    free_modreqs (np->dirmod_reqs);
+  if (np->filemod_reqs)
+    free_modreqs (np->filemod_reqs);
 
   assert (!np->sockaddr);
 
+  pthread_mutex_unlock(&np->lock);
+  pthread_mutex_destroy(&np->lock);
   diskfs_node_norefs (np);
-  spin_unlock (&diskfs_node_refcnt_lock);
 }
-
-      
