@@ -18,11 +18,16 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */
 
-#include "trans.h"
-#include <unistd.h>
 #include <assert.h>
-#include <string.h>
 #include <hurd/fsys.h>
+#include <hurd/paths.h>
+#include <hurd/ports.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
+
+#include "fshelp.h"
 
 error_t
 fshelp_fetch_root (struct transbox *box, void *cookie,
@@ -72,10 +77,10 @@ fshelp_fetch_root (struct transbox *box, void *cookie,
 	  /* MAKE_SEND is safe here because we destroy REND ourselves. */
 	  err = io_reauthenticate (port, rend,
 				   MACH_MSG_TYPE_MAKE_SEND);
-	  mach_port_deallocate (mach_task_self (), port);
 	  if (! err)
 	    err = auth_user_authenticate (newauth, rend,
 					  MACH_MSG_TYPE_MAKE_SEND, &ret);
+	  mach_port_deallocate (mach_task_self (), port);
 	  if (err)
 	    ret = MACH_PORT_NULL;
 
@@ -131,7 +136,7 @@ fshelp_fetch_root (struct transbox *box, void *cookie,
 
       fds[STDERR_FILENO] = reauth (getdport (STDERR_FILENO));
 
-      err = fshelp_start_translator_long (fetch_underlying, NULL,
+      err = fshelp_start_translator_long (fetch_underlying, cookie,
 					  argz, argz, argz_len,
 					  fds, MACH_MSG_TYPE_COPY_SEND,
 					  STDERR_FILENO + 1,
@@ -186,10 +191,49 @@ fshelp_fetch_root (struct transbox *box, void *cookie,
   if ((err == MACH_SEND_INVALID_DEST || err == MIG_SERVER_DIED)
       && control == box->active)
     fshelp_set_active (box, MACH_PORT_NULL, 0);
-  mach_port_deallocate (mach_task_self (), control);
+  else
+    mach_port_deallocate (mach_task_self (), control);
 
   if (err == MACH_SEND_INVALID_DEST || err == MIG_SERVER_DIED)
     goto start_over;
 
   return err;
 }
+
+/* A callback function for short-circuited translators.  S_ISLNK and
+   S_IFSOCK must be handled elsewhere.  */
+error_t
+fshelp_short_circuited_callback1 (void *cookie1, void *cookie2,
+				  uid_t *uid, gid_t *gid,
+				  char **argz, size_t *argz_len)
+{
+  struct fshelp_stat_cookie2 *statc = cookie2;
+
+  switch (*statc->modep & S_IFMT)
+    {
+    case S_IFCHR:
+    case S_IFBLK:
+      if (asprintf (argz, "%s%c%d%c%d",
+		    (S_ISCHR (*statc->modep)
+		     ? _HURD_CHRDEV : _HURD_BLKDEV),
+		    0, major (statc->statp->st_rdev),
+		    0, minor (statc->statp->st_rdev)) < 0)
+	return ENOMEM;
+      *argz_len = strlen (*argz) + 1;
+      *argz_len += strlen (*argz + *argz_len) + 1;
+      *argz_len += strlen (*argz + *argz_len) + 1;
+      break;
+    case S_IFIFO:
+      if (asprintf (argz, "%s", _HURD_FIFO) < 0)
+	return ENOMEM;
+      *argz_len = strlen (*argz) + 1;
+      break;
+    default:
+      return ENOENT;
+    }
+
+  *uid = statc->statp->st_uid;
+  *gid = statc->statp->st_gid;
+
+  return 0;
+}
