@@ -1,5 +1,6 @@
-/* 
-   Copyright (C) 1996 Free Software Foundation, Inc.
+/*
+   Copyright (C) 1996, 1997, 1999, 2001, 2013, 2014
+     Free Software Foundation, Inc.
    Written by Michael I. Bushnell, p/BSG.
 
    This file is part of the GNU Hurd.
@@ -19,11 +20,10 @@
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA. */
 
 #include "netfs.h"
+#include "fs_S.h"
 #include <hurd/paths.h>
 #include <hurd/fsys.h>
 
-#define makedev(maj,min) ((((maj)&0xFF)<<8)+((min)&0xFF))
-
 error_t
 netfs_S_file_set_translator (struct protid *user,
 			     int passive_flags, int active_flags,
@@ -37,17 +37,18 @@ netfs_S_file_set_translator (struct protid *user,
 
   if (!user)
     return EOPNOTSUPP;
-  
+
   if (!(passive_flags & FS_TRANS_SET) && !(active_flags & FS_TRANS_SET))
     return 0;
-  
-  if (passive && passive[passivelen - 1])
+
+  if (passivelen && passive[passivelen - 1])
     return EINVAL;
-  
+
   np = user->po->np;
-  mutex_lock (&np->lock);
-  
-  if (active_flags & FS_TRANS_SET)
+  pthread_mutex_lock (&np->lock);
+
+  if (active_flags & FS_TRANS_SET
+      && ! (active_flags & FS_TRANS_ORPHAN))
     {
       /* Validate--user must be owner */
       err = netfs_validate_stat (np, user->user);
@@ -57,20 +58,20 @@ netfs_S_file_set_translator (struct protid *user,
       err = fshelp_isowner (&np->nn_stat, user->user);
       if (err)
 	goto out;
-      
+
       err = fshelp_fetch_control (&np->transbox, &control);
       if (err)
 	goto out;
-      
-      if (control != MACH_PORT_NULL 
+
+      if (control != MACH_PORT_NULL
 	  && (active_flags & FS_TRANS_EXCL) == 0)
 	{
-	  mutex_unlock (&np->lock);
+	  pthread_mutex_unlock (&np->lock);
 	  err = fsys_goaway (control, killtrans_flags);
 	  if (err && err != MIG_SERVER_DIED && err != MACH_SEND_INVALID_DEST)
 	    return err;
 	  err = 0;
-	  mutex_lock (&np->lock);
+	  pthread_mutex_lock (&np->lock);
 	}
     }
 
@@ -78,12 +79,12 @@ netfs_S_file_set_translator (struct protid *user,
       && (passive_flags & FS_TRANS_EXCL))
     {
       err = netfs_validate_stat (np, user->user);
-      if (!err && np->istranslated)
+      if (!err && (np->nn_stat.st_mode & S_IPTRANS))
 	err = EBUSY;
       if (err)
 	goto out;
     }
-  
+
   if (active_flags & FS_TRANS_SET)
     {
       err = fshelp_set_active (&np->transbox, active,
@@ -91,7 +92,7 @@ netfs_S_file_set_translator (struct protid *user,
       if (err)
 	goto out;
     }
-  
+
   if (passive_flags & FS_TRANS_SET)
     {
       mode_t newmode = 0;
@@ -110,12 +111,12 @@ netfs_S_file_set_translator (struct protid *user,
 	  else if (!strcmp (passive, _HURD_IFSOCK))
 	    newmode = S_IFSOCK;
 	}
-      
+
       switch (newmode)
 	{
 	  int major, minor;
 	  char *arg;
-	  
+
 	case S_IFBLK:
 	case S_IFCHR:
 	  /* Find the device number from the arguments
@@ -124,35 +125,35 @@ netfs_S_file_set_translator (struct protid *user,
 	  assert (arg <= passive + passivelen);
 	  if (arg == passive + passivelen)
 	    {
-	      mutex_unlock (&np->lock);
+	      pthread_mutex_unlock (&np->lock);
 	      return EINVAL;
 	    }
 	  major = strtol (arg, 0, 0);
-	  
+
 	  arg = arg + strlen (arg) + 1;
 	  assert (arg < passive + passivelen);
 	  if (arg == passive + passivelen)
 	    {
-	      mutex_unlock (&np->lock);
+	      pthread_mutex_unlock (&np->lock);
 	      return EINVAL;
 	    }
 	  minor = strtol (arg, 0, 0);
-	  
+
 	  err = netfs_attempt_mkdev (user->user, np,
 				     newmode, makedev (major, minor));
 	  if (err == EOPNOTSUPP)
 	    goto fallback;
 	  break;
-	  
+
 	case S_IFLNK:
 	  arg = passive + strlen (passive) + 1;
 	  assert (arg <= passive + passivelen);
 	  if (arg == passive + passivelen)
 	    {
-	      mutex_unlock (&np->lock);
+	      pthread_mutex_unlock (&np->lock);
 	      return EINVAL;
 	    }
-	  
+
 	  err = netfs_attempt_mksymlink (user->user, np, arg);
 	  if (err == EOPNOTSUPP)
 	    goto fallback;
@@ -167,16 +168,19 @@ netfs_S_file_set_translator (struct protid *user,
 	  if (err == EOPNOTSUPP)
 	    goto fallback;
 	  break;
-	  
+
 	case 0:
 	fallback:
-	  err = netfs_set_translator (user->user, np, 
+	  err = netfs_set_translator (user->user, np,
 				      passive, passivelen);
 	  break;
 	}
     }
-  
+
+  if (! err && user->po->path && active_flags & FS_TRANS_SET)
+    err = fshelp_set_active_translator (&user->pi, user->po->path, active);
+
  out:
-  mutex_unlock (&np->lock);
+  pthread_mutex_unlock (&np->lock);
   return err;
 }
