@@ -23,21 +23,21 @@
 #include "fs_S.h"
 #include "ifsock_S.h"
 
-error_t
+error_t __attribute__((weak))
 netfs_S_file_notice_changes (struct protid *user,
 			     mach_port_t port)
 {
   return EOPNOTSUPP;
 }
 
-error_t
+error_t __attribute__((weak))
 netfs_S_file_getfh (struct protid *user,
 		    char **data, mach_msg_type_number_t *ndata)
 {
   return EOPNOTSUPP;
 }
 
-error_t
+error_t __attribute__((weak))
 netfs_S_ifsock_getsockaddr (struct protid *user,
 			    mach_port_t *address)
 {
