@@ -1,5 +1,5 @@
 /* Implementation of memory_object_terminate for pager library
-   Copyright (C) 1994, 1995, 1996 Free Software Foundation
+   Copyright (C) 1994, 1995, 1996, 1999, 2000 Free Software Foundation
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -22,35 +22,31 @@
 /* Implement the object termination call from the kernel as described
    in <mach/memory_object.defs>. */
 kern_return_t
-_pager_seqnos_memory_object_terminate (mach_port_t object, 
-				       mach_port_seqno_t seqno,
+_pager_S_memory_object_terminate (struct pager *p,
 				       mach_port_t control,
 				       mach_port_t name)
 {
-  struct pager *p;
-  
-  p = ports_lookup_port (0, object, _pager_class);
-  if (!p)
+  if (!p
+      || p->port.class != _pager_class)
     return EOPNOTSUPP;
 
-  mutex_lock (&p->interlock);
-  _pager_wait_for_seqno (p, seqno);
-  
+  pthread_mutex_lock (&p->interlock);
+
   if (control != p->memobjcntl)
     {
-      printf ("incg terminate: wrong control port");
+      printf ("incg terminate: wrong control port\n");
       goto out;
     }
   if (name != p->memobjname)
     {
-      printf ("incg terminate: wrong name port");
+      printf ("incg terminate: wrong name port\n");
       goto out;
     }
 
   while (p->noterm)
     {
       p->termwaiting = 1;
-      condition_wait (&p->wakeup, &p->interlock);
+      pthread_cond_wait (&p->wakeup, &p->interlock);
     }
 
   /* Destry the ports we received; mark that in P so that it doesn't bother
@@ -77,9 +73,7 @@ _pager_seqnos_memory_object_terminate (mach_port_t object,
 #endif
 
  out:
-  _pager_release_seqno (p, seqno);
-  mutex_unlock (&p->interlock);
-  ports_port_deref (p);
+  pthread_mutex_unlock (&p->interlock);
 
   return 0;
 }
@@ -108,7 +102,7 @@ _pager_free_structure (struct pager *p)
     }
 
   if (wakeup)
-    condition_broadcast (&p->wakeup);
+    pthread_cond_broadcast (&p->wakeup);
 
   if (p->memobjcntl != MACH_PORT_NULL)
     {
@@ -124,7 +118,7 @@ _pager_free_structure (struct pager *p)
   /* Free the pagemap */
   if (p->pagemapsize)
     {
-      vm_deallocate (mach_task_self (), (u_int)p->pagemap, p->pagemapsize);
+      munmap (p->pagemap, p->pagemapsize * sizeof (* p->pagemap));
       p->pagemapsize = 0;
       p->pagemap = 0;
     }
