@@ -1,5 +1,5 @@
-/* 
-   Copyright (C) 1994 Free Software Foundation
+/*
+   Copyright (C) 1994,99,2002 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -16,13 +16,13 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */
 
 #include "priv.h"
-#include "fs_S.h"
 
 kern_return_t
 trivfs_S_dir_readdir (struct trivfs_protid *cred,
 		      mach_port_t reply, mach_msg_type_name_t reply_type,
 		      char **data,
-		      u_int *datalen,
+		      size_t *datalen,
+		      boolean_t *data_dealloc,
 		      int entry,
 		      int nentries,
 		      vm_size_t bufsiz,
