@@ -45,9 +45,6 @@ trivfs_begin_using_protid (mach_port_t port)
       for (i = 0; i < trivfs_num_dynamic_protid_port_classes; i++)
 	if (pi->class == trivfs_dynamic_protid_port_classes[i])
 	  return (struct trivfs_protid *) pi;
-      for (i = 0; i < trivfs_protid_nportclasses; i++)
-	if (pi->class == trivfs_protid_portclasses[i])
-	  return (struct trivfs_protid *) pi;
       ports_port_deref (pi);
     }
 
@@ -65,9 +62,6 @@ trivfs_begin_using_protid_payload (unsigned long payload)
       for (i = 0; i < trivfs_num_dynamic_protid_port_classes; i++)
 	if (pi->class == trivfs_dynamic_protid_port_classes[i])
 	  return (struct trivfs_protid *) pi;
-      for (i = 0; i < trivfs_protid_nportclasses; i++)
-	if (pi->class == trivfs_protid_portclasses[i])
-	  return (struct trivfs_protid *) pi;
       ports_port_deref (pi);
     }
 
@@ -81,6 +75,12 @@ trivfs_end_using_protid (struct trivfs_protid *cred)
     ports_port_deref (cred);
 }
 
+static inline mach_port_t __attribute__ ((unused))
+trivfs_convert_to_port(struct trivfs_protid *protid)
+{
+  return protid->pi.port_right;
+}
+
 static inline struct trivfs_control * __attribute__ ((unused))
 trivfs_begin_using_control (mach_port_t port)
 {
@@ -92,9 +92,6 @@ trivfs_begin_using_control (mach_port_t port)
       for (i = 0; i < trivfs_num_dynamic_control_port_classes; i++)
 	if (pi->class == trivfs_dynamic_control_port_classes[i])
 	  return (struct trivfs_control *) pi;
-      for (i = 0; i < trivfs_cntl_nportclasses; i++)
-	if (pi->class == trivfs_cntl_portclasses[i])
-	  return (struct trivfs_control *) pi;
       ports_port_deref (pi);
     }
 
@@ -112,9 +109,6 @@ trivfs_begin_using_control_payload (unsigned long payload)
       for (i = 0; i < trivfs_num_dynamic_control_port_classes; i++)
 	if (pi->class == trivfs_dynamic_control_port_classes[i])
 	  return (struct trivfs_control *) pi;
-      for (i = 0; i < trivfs_cntl_nportclasses; i++)
-	if (pi->class == trivfs_cntl_portclasses[i])
-	  return (struct trivfs_control *) pi;
       ports_port_deref (pi);
     }
 
