@@ -1,5 +1,5 @@
 /* 
-   Copyright (C) 1994 Free Software Foundation
+   Copyright (C) 1994, 1999, 2007 Free Software Foundation
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -20,33 +20,37 @@
 error_t
 trivfs_set_atime (struct trivfs_control *cntl)
 {
+  error_t err;
   struct stat st;
   time_value_t atime;
   time_value_t mtime;
-  
-  io_stat (cntl->underlying, &st);
-  mtime.seconds = st.st_mtime;
-  mtime.microseconds = st.st_mtime_usec;
-  host_get_time (mach_host_self (), &atime);
-  file_utimes (cntl->underlying, atime, mtime);
-  return 0;
+
+  err = io_stat (cntl->underlying, &st);
+  if (err)
+    return err;
+
+  mtime.seconds = st.st_mtim.tv_sec;
+  mtime.microseconds = st.st_mtim.tv_nsec / 1000;
+  atime.microseconds = -1;
+
+  return file_utimes (cntl->underlying, atime, mtime);
 }
 
 error_t
 trivfs_set_mtime (struct trivfs_control *cntl)
 {
+  error_t err;
   struct stat st;
   time_value_t atime;
   time_value_t mtime;
 
-  io_stat (cntl->underlying, &st);
-  atime.seconds = st.st_atime;
-  atime.microseconds = st.st_atime_usec;
-  host_get_time (mach_host_self (), &mtime);
-  file_utimes (cntl->underlying, atime, mtime);
-  return 0;
+  err = io_stat (cntl->underlying, &st);
+  if (err)
+    return err;
+
+  atime.seconds = st.st_atim.tv_sec;
+  atime.microseconds = st.st_atim.tv_nsec / 1000;
+  mtime.microseconds = -1;
+
+  return file_utimes (cntl->underlying, atime, mtime);
 }
-
-
-
-  
