@@ -1,5 +1,5 @@
-/* 
-   Copyright (C) 1996 Free Software Foundation, Inc.
+/*
+   Copyright (C) 1996,98,2002 Free Software Foundation, Inc.
    Written by Michael I. Bushnell, p/BSG.
 
    This file is part of the GNU Hurd.
@@ -18,14 +18,12 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA. */
 
-typedef int bool_t;
-typedef int enum_t;
-
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <errno.h>
 #include <netinet/in.h>
-#include <cthreads.h>
+#include <pthread.h>
+#include <rpc/types.h>
 #include "../nfs/nfs-spec.h" /* XXX */
 #include <hurd/fs.h>
 
@@ -35,7 +33,7 @@ typedef int enum_t;
 #define REPLY_KEEP_TIMEOUT 120	/* two minutes */
 #define MAXIOSIZE 10240
 
-struct idspec 
+struct idspec
 {
   struct idspec *next, **prevp;
   int nuids, ngids;
@@ -44,10 +42,15 @@ struct idspec
   int references;
 };
 
+union cache_handle_array {
+  char array[NFS2_FHSIZE];
+  int fs;
+};
+
 struct cache_handle
 {
   struct cache_handle *next, **prevp;
-  char handle[NFS2_FHSIZE];
+  union cache_handle_array handle;
   struct idspec *ids;
   file_t port;
   time_t lastuse;
@@ -57,7 +60,7 @@ struct cache_handle
 struct cached_reply
 {
   struct cached_reply *next, **prevp;
-  struct mutex lock;
+  pthread_mutex_t lock;
   struct sockaddr_in source;
   int xid;
   time_t lastuse;
@@ -74,11 +77,11 @@ struct procedure
   int process_error;
 };
 
-struct proctable 
+struct proctable
 {
   int min;
   int max;
-  struct procedure procs[0];
+  struct procedure procs[];
 };
 
 volatile struct mapped_time_value *mapped_time;
@@ -111,7 +114,7 @@ void release_cached_reply (struct cached_reply *cr);
 void scan_replies (void);
 
 /* loop.c */
-void server_loop (int);
+void * server_loop (void *);
 
 /* ops.c */
 extern struct proctable nfs2table, mounttable, pmaptable;
