@@ -21,6 +21,7 @@
 #include <mach/vm_param.h>
 #include <mach/vm_statistics.h>
 #include <mach/vm_cache_statistics.h>
+#include "default_pager_U.h"
 #include <mach/default_pager.h>
 #include <mach_debug/mach_debug_types.h>
 #include <hurd/paths.h>
@@ -271,27 +272,37 @@ rootdir_gc_meminfo (void *hook, char **contents, ssize_t *contents_len)
   struct vm_statistics vmstats;
   struct vm_cache_statistics cache_stats;
   default_pager_info_t swap;
+  FILE *m;
   error_t err;
 
+  m = open_memstream (contents, (size_t *) contents_len);
+  if (m == NULL)
+    {
+      err = ENOMEM;
+      goto out;
+    }
+
   err = vm_statistics (mach_task_self (), &vmstats);
   if (err)
-    return EIO;
+    {
+      err = EIO;
+      goto out;
+    }
 
   err = vm_cache_statistics (mach_task_self (), &cache_stats);
   if (err)
-    return EIO;
+    {
+      err = EIO;
+      goto out;
+    }
 
   cnt = HOST_BASIC_INFO_COUNT;
   err = host_info (mach_host_self (), HOST_BASIC_INFO, (host_info_t) &hbi, &cnt);
   if (err)
-    return err;
-
-  err = get_swapinfo (&swap);
-  if (err)
-    return err;
+    goto out;
 
   assert (cnt == HOST_BASIC_INFO_COUNT);
-  *contents_len = asprintf (contents,
+  fprintf (m,
       "MemTotal: %14lu kB\n"
       "MemFree:  %14lu kB\n"
       "Buffers:  %14lu kB\n"
@@ -299,8 +310,6 @@ rootdir_gc_meminfo (void *hook, char **contents, ssize_t *contents_len)
       "Active:   %14lu kB\n"
       "Inactive: %14lu kB\n"
       "Mlocked:  %14lu kB\n"
-      "SwapTotal:%14lu kB\n"
-      "SwapFree: %14lu kB\n"
       ,
       (long unsigned) hbi.memory_size / 1024,
       (long unsigned) vmstats.free_count * PAGE_SIZE / 1024,
@@ -308,11 +317,23 @@ rootdir_gc_meminfo (void *hook, char **contents, ssize_t *contents_len)
       (long unsigned) cache_stats.cache_count * PAGE_SIZE / 1024,
       (long unsigned) vmstats.active_count * PAGE_SIZE / 1024,
       (long unsigned) vmstats.inactive_count * PAGE_SIZE / 1024,
-      (long unsigned) vmstats.wire_count * PAGE_SIZE / 1024,
+      (long unsigned) vmstats.wire_count * PAGE_SIZE / 1024);
+
+  err = get_swapinfo (&swap);
+  if (err)
+    /* This is not fatal, we just omit the information.  */
+    err = 0;
+  else
+    fprintf (m,
+      "SwapTotal:%14lu kB\n"
+      "SwapFree: %14lu kB\n"
+      ,
       (long unsigned) swap.dpi_total_space / 1024,
       (long unsigned) swap.dpi_free_space / 1024);
 
-  return 0;
+ out:
+  fclose (m);
+  return err;
 }
 
 static error_t
@@ -432,7 +453,7 @@ rootdir_gc_slabinfo (void *hook, char **contents, ssize_t *contents_len)
   if (err)
     return err;
 
-  m = open_memstream (contents, contents_len);
+  m = open_memstream (contents, (size_t *) contents_len);
   if (m == NULL)
     {
       err = ENOMEM;
@@ -446,12 +467,11 @@ rootdir_gc_slabinfo (void *hook, char **contents, ssize_t *contents_len)
 
   for (i = 0; i < cache_info_count; i++)
     {
-      mem_usage = (cache_info[i].nr_slabs * cache_info[i].slab_size)
-	>> 10;
+      mem_usage =
+	(cache_info[i].nr_slabs * cache_info[i].slab_size) >> 10;
       mem_total += mem_usage;
-      mem_reclaimable = (cache_info[i].flags & CACHE_FLAGS_NO_RECLAIM)
-	? 0 : (cache_info[i].nr_free_slabs
-	       * cache_info[i].slab_size) >> 10;
+      mem_reclaimable =
+	(cache_info[i].nr_free_slabs * cache_info[i].slab_size) >> 10;
       mem_total_reclaimable += mem_reclaimable;
       fprintf (m,
                "%-21s %04x %7zu %3zuk  %4lu %6lu %6lu %7zuk %10zuk\n",
@@ -467,8 +487,8 @@ rootdir_gc_slabinfo (void *hook, char **contents, ssize_t *contents_len)
   fclose (m);
 
  out:
-  vm_deallocate (mach_task_self (),
-                 cache_info, cache_info_count * sizeof *cache_info);
+  vm_deallocate (mach_task_self (), (vm_address_t) cache_info,
+                 cache_info_count * sizeof *cache_info);
   return err;
 }
 
@@ -481,7 +501,7 @@ rootdir_gc_filesystems (void *hook, char **contents, ssize_t *contents_len)
   glob_t matches;
   FILE *m;
 
-  m = open_memstream (contents, contents_len);
+  m = open_memstream (contents, (size_t *) contents_len);
   if (m == NULL)
     return errno;
 
@@ -522,6 +542,56 @@ rootdir_gc_filesystems (void *hook, char **contents, ssize_t *contents_len)
   fclose (m);
   return err;
 }
+
+static error_t
+rootdir_gc_swaps (void *hook, char **contents, ssize_t *contents_len)
+{
+  mach_port_t defpager;
+  error_t err = 0;
+  FILE *m;
+  vm_size_t *free = NULL;
+  size_t nfree = 0;
+  vm_size_t *size = NULL;
+  size_t nsize = 0;
+  char *names = NULL, *name;
+  size_t names_len = 0;
+  size_t i;
+
+  m = open_memstream (contents, (size_t *) contents_len);
+  if (m == NULL)
+    return errno;
+
+  defpager = file_name_lookup (_SERVERS_DEFPAGER, O_READ, 0);
+  if (defpager == MACH_PORT_NULL)
+    {
+      err = errno;
+      goto out_fclose;
+    }
+
+  err = default_pager_storage_info (defpager, &size, &nsize, &free, &nfree,
+				    &names, &names_len);
+  if (err)
+    goto out;
+
+  fprintf(m, "Filename\tType\t\tSize\tUsed\tPriority\n");
+  name = names;
+  for (i = 0; i < nfree; i++)
+    {
+      fprintf (m, "%s\tpartition\t%zu\t%zu\t-1\n",
+	       name, size[i] >> 10, (size[i] - free[i]) >> 10);
+      name = argz_next (names, names_len, name);
+    }
+
+  vm_deallocate (mach_task_self(), (vm_offset_t) free, nfree * sizeof(*free));
+  vm_deallocate (mach_task_self(), (vm_offset_t) size, nsize * sizeof(*size));
+  vm_deallocate (mach_task_self(), (vm_offset_t) names, names_len);
+
+out:
+  mach_port_deallocate (mach_task_self (), defpager);
+out_fclose:
+  fclose (m);
+  return err;
+}
 
 /* Glue logic and entries table */
 
@@ -574,7 +644,7 @@ rootdir_make_translated_node (void *dir_hook, const void *entry_hook)
       return np;
     }
 
-  np = procfs_make_node (entry_hook, entry_hook);
+  np = procfs_make_node (entry_hook, (void *) entry_hook);
   if (np == NULL)
     return NULL;
 
@@ -704,6 +774,13 @@ static const struct procfs_dir_entry rootdir_entries[] = {
       .cleanup_contents = procfs_cleanup_contents_with_free,
     },
   },
+  {
+    .name = "swaps",
+    .hook = & (struct procfs_node_ops) {
+      .get_contents = rootdir_gc_swaps,
+      .cleanup_contents = procfs_cleanup_contents_with_free,
+    },
+  },
 #ifdef PROFILE
   /* In order to get a usable gmon.out file, we must apparently use exit(). */
   {
