@@ -879,6 +879,23 @@ init_stdarrays ()
 /* Frobnicate the kernel task and the proc server's idea of it (PID 2),
    so the kernel command line can be read as for a normal Hurd process.  */
 
+void
+dump_processes (void)
+{
+  pid_t pid;
+  for (pid = 1; pid < 100; pid++)
+    {
+      char args[256], *buffer = args;
+      size_t len = sizeof args;
+      if (proc_getprocargs (procserver, pid, &buffer, &len) == 0)
+        {
+          fprintf (stderr, "pid%d\t%s\n", (int) pid, buffer);
+          if (buffer != args)
+            vm_deallocate (mach_task_self (), (vm_offset_t) buffer, len);
+        }
+    }
+}
+
 void
 frob_kernel_process (void)
 {
@@ -900,6 +917,15 @@ frob_kernel_process (void)
       error (0, err, "cannot get kernel task port");
       return;
     }
+
+  /* Make the kernel our child.  */
+  err = proc_child (procserver, task);
+  if (err)
+    {
+      error (0, err, "cannot make the kernel our child");
+      dump_processes ();
+    }
+
   err = proc_task2proc (procserver, task, &proc);
   if (err)
     {
@@ -910,7 +936,9 @@ frob_kernel_process (void)
 
   /* Mark the kernel task as an essential task so that we or the proc server
      never want to task_terminate it.  */
-  proc_mark_important (proc);
+  err = proc_mark_important (proc);
+  if (err)
+    error (0, err, "cannot mark the kernel as important");
 
   err = record_essential_task ("kernel", task);
   assert_perror (err);
@@ -1194,6 +1222,8 @@ launch_something (const char *why)
 	  if (start_child (tries[try++], NULL) == 0)
 	    return;
 	}
+      else
+	try++;
     }
 
   crash_system ();
