@@ -1,8 +1,7 @@
 /* A translator for fifos
 
-   Copyright (C) 1995, 1996 Free Software Foundation, Inc.
-
-   Written by Miles Bader <miles@gnu.ai.mit.edu>
+   Copyright (C) 1995,96,97,98,2000,02 Free Software Foundation, Inc.
+   Written by Miles Bader <miles@gnu.org>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -27,7 +26,7 @@
 #include <fcntl.h>
 #include <assert.h>
 
-#include <cthreads.h>
+#include <pthread.h>
 #include <hurd.h>
 #include <argz.h>
 #include <hurd/fshelp.h>
@@ -37,8 +36,16 @@
 #include <hurd/pipe.h>
 #include <hurd/paths.h>
 
+#include <version.h>
+
+#include "libtrivfs/trivfs_fs_S.h"
+#include "libtrivfs/trivfs_fsys_S.h"
+#include "libtrivfs/trivfs_io_S.h"
+
 #define DEFAULT_SERVER _SERVERS "fifo";
 
+const char *argp_program_version = STANDARD_HURD_VERSION (new-fifo);
+
 struct port_bucket *port_bucket;
 struct port_class *fifo_port_class, *server_port_class, *fsys_port_class;
 
@@ -82,9 +89,9 @@ struct fifo_trans
   /* The current fifo that new opens will see, or NULL if there is none.  */
   struct pipe *active_fifo;
   /* Lock this when changing ACTIVE_FIFO.  */
-  struct mutex active_fifo_lock;
+  pthread_mutex_t active_fifo_lock;
   /* Signal this when ACTIVE_FIFO may have changed.  */
-  struct condition active_fifo_changed;
+  pthread_cond_t active_fifo_changed;
 };
 
 /* Return a new FIFO_TRANS in TRANS, initializing it from FROM if it's
@@ -95,8 +102,8 @@ fifo_trans_create (struct fifo_trans *from, struct fifo_trans **trans)
   struct fifo_trans *new = malloc (sizeof (struct fifo_trans));
 
   new->server = 0;
-  mutex_init (&new->active_fifo_lock);
-  condition_init (&new->active_fifo_changed);
+  pthread_mutex_init (&new->active_fifo_lock, NULL);
+  pthread_cond_init (&new->active_fifo_changed, NULL);
 
   new->parent = from;
 
@@ -165,18 +172,11 @@ fifo_trans_parse_args (struct fifo_trans *trans, int argc, char **argv,
 	}
       return 0;
     }
-  struct argp argp = {options, parse_opt};
+  struct argp argp = {options, parse_opt, 0, "A translator for fifos." };
   return argp_parse (&argp, argc, argv, print_errs ? 0 : ARGP_SILENT, 0, 0);
 }
 
-/* ---------------------------------------------------------------- */
-
-struct port_class *trivfs_protid_portclasses[1];
-struct port_class *trivfs_cntl_portclasses[1];
-int trivfs_protid_nportclasses = 1;
-int trivfs_cntl_nportclasses = 1;
-
-void
+int
 main (int argc, char **argv)
 {
   error_t err;
@@ -208,14 +208,21 @@ main (int argc, char **argv)
 	exit (0);
     }
 
-  port_bucket = ports_create_bucket ();
-  fifo_port_class =  ports_create_class (trivfs_clean_protid, 0);
-  server_port_class =  ports_create_class (trivfs_clean_protid, 0);
-  fsys_port_class = ports_create_class (clean_fsys, 0);
+  err = trivfs_add_port_bucket (&port_bucket);
+  if (err)
+    error (1, 0, "error creating port bucket");
 
-  trivfs_protid_portclasses[0] = fifo_port_class;
-  trivfs_protid_portclasses[1] = server_port_class;
-  trivfs_cntl_portclasses[0] = fsys_port_class;
+  err = trivfs_add_control_port_class (&fsys_port_class);
+  if (err)
+    error (1, 0, "error creating control port class");
+
+  err = trivfs_add_protid_port_class (&fifo_port_class);
+  if (err)
+    error (1, 0, "error creating protid port class");
+
+    err = trivfs_add_protid_port_class (&server_port_class);
+  if (err)
+    error (1, 0, "error creating protid port class");
 
   /* Reply to our parent */
   fifo_trans_start (trans, bootstrap);
@@ -226,11 +233,11 @@ main (int argc, char **argv)
       ports_enable_class (fifo_port_class);
       ports_manage_port_operations_multithread (port_bucket,
 						trivfs_demuxer,
-						30*1000, 5*60*1000, 0, 0);
+						30*1000, 5*60*1000, 0);
     }
   while (ports_count_class (fifo_port_class) > 0);
 
-  exit (0);
+  return 0;
 }
 
 /* ---------------------------------------------------------------- */
@@ -242,19 +249,21 @@ fifo_trans_open (struct fifo_trans *trans, int flags, void **hook)
 
   if (flags & (O_READ | O_WRITE))
     {
-      mutex_lock (&trans->active_fifo_lock);
+      pthread_mutex_lock (&trans->active_fifo_lock);
 
 /* Wait until the active fifo has changed so that CONDITION is true.  */
-#define WAIT(condition, noblock_err)					     \
-  while (!err && !(condition)) 						     \
-    if (flags & O_NONBLOCK) 						     \
-      {			 						     \
-        err = noblock_err; 						     \
-        break;		 						     \
-      }			 						     \
-    else if (hurd_condition_wait (&trans->active_fifo_changed,		     \
-				  &trans->active_fifo_lock))		     \
-      err = EINTR;
+#define WAIT(condition, noblock_err)					      \
+  while (!err && !(condition))						      \
+    {									      \
+      if (flags & O_NONBLOCK)						      \
+	{								      \
+	  err = noblock_err;						      \
+	  break;							      \
+	}								      \
+      else if (pthread_hurd_cond_wait_np (&trans->active_fifo_changed,	      \
+					  &trans->active_fifo_lock))	      \
+	err = EINTR;							      \
+    }
 
       if (flags & O_READ)
 	/* When opening for read, what we do depends on what mode this server
@@ -276,7 +285,7 @@ fifo_trans_open (struct fifo_trans *trans, int flags, void **hook)
 	  if (!err)
 	    {
 	      pipe_add_reader (trans->active_fifo);
-	      condition_broadcast (&trans->active_fifo_changed);
+	      pthread_cond_broadcast (&trans->active_fifo_changed);
 	      /* We'll unlock ACTIVE_FIFO_LOCK below; the writer code won't
 		 make us block because we've ensured that there's a reader
 		 for it.  */
@@ -290,7 +299,7 @@ fifo_trans_open (struct fifo_trans *trans, int flags, void **hook)
 		    {
 		      pipe_remove_reader (trans->active_fifo);
 		      trans->active_fifo = NULL;
-		      condition_broadcast (&trans->active_fifo_changed);
+		      pthread_cond_broadcast (&trans->active_fifo_changed);
 		    }
 		}
 	      else
@@ -318,14 +327,14 @@ fifo_trans_open (struct fifo_trans *trans, int flags, void **hook)
 	  if (!err)
 	    {
 	      pipe_add_writer (trans->active_fifo);
-	      condition_broadcast (&trans->active_fifo_changed);
+	      pthread_cond_broadcast (&trans->active_fifo_changed);
 	    }
 	}
 
       *hook = trans->active_fifo;
     }
 
-  mutex_unlock (&trans->active_fifo_lock);
+  pthread_mutex_unlock (&trans->active_fifo_lock);
 
   return err;
 }
@@ -340,7 +349,7 @@ fifo_trans_close (struct fifo_trans *trans, struct trivfs_peropen *po)
   if (!pipe)
     return;
 
-  mutex_lock (&trans->active_fifo_lock);
+  pthread_mutex_lock (&trans->active_fifo_lock);
   was_active = (trans->active_fifo == pipe);
 
   if (was_active)
@@ -348,7 +357,7 @@ fifo_trans_close (struct fifo_trans *trans, struct trivfs_peropen *po)
     going_away = ((flags & O_READ) && pipe->readers == 1);
   else
     /* Let others have their fun.  */
-    mutex_unlock (&trans->active_fifo_lock);
+    pthread_mutex_unlock (&trans->active_fifo_lock);
 
   if (flags & O_READ)
     pipe_remove_reader (pipe);
@@ -360,8 +369,8 @@ fifo_trans_close (struct fifo_trans *trans, struct trivfs_peropen *po)
     {
       if (going_away)
 	trans->active_fifo = NULL;
-      condition_broadcast (&trans->active_fifo_changed);
-      mutex_unlock (&trans->active_fifo_lock);
+      pthread_cond_broadcast (&trans->active_fifo_changed);
+      pthread_mutex_unlock (&trans->active_fifo_lock);
     }
 }
 
@@ -418,10 +427,10 @@ trivfs_modify_stat (struct trivfs_protid *cred, struct stat *st)
 
       if (pipe)
 	{
-	  mutex_lock (&pipe->lock);
+	  pthread_mutex_lock (&pipe->lock);
 	  st->st_size = pipe_readable (pipe, 1);
 	  st->st_blocks = st->st_size >> 9;
-	  mutex_unlock (&pipe->lock);
+	  pthread_mutex_unlock (&pipe->lock);
 	}
       else
 	st->st_size = st->st_blocks = 0;
@@ -497,13 +506,14 @@ trivfs_goaway (struct trivfs_control *fsys, int flags)
    mapping; they will set none of the ports and return an error.  Such
    objects can still be accessed by io_read and io_write.  */
 error_t
-trivfs_S_io_map(struct trivfs_protid *cred,
-		memory_object_t *rdobj,
-		mach_msg_type_name_t *rdtype,
-		memory_object_t *wrobj,
-		mach_msg_type_name_t *wrtype)
+trivfs_S_io_map (struct trivfs_protid *cred,
+		 mach_port_t reply, mach_msg_type_name_t replytype,
+		 memory_object_t *rdobj,
+		 mach_msg_type_name_t *rdtype,
+		 memory_object_t *wrobj,
+		 mach_msg_type_name_t *wrtype)
 {
-  return EINVAL;
+  return EOPNOTSUPP;
 }
 
 /* ---------------------------------------------------------------- */
@@ -527,10 +537,10 @@ trivfs_S_io_read (struct trivfs_protid *cred,
     {
       struct pipe *pipe = cred->po->hook;
       assert (pipe);
-      mutex_lock (&pipe->lock);
+      pthread_mutex_lock (&pipe->lock);
       err = pipe_read (pipe, cred->po->openmodes & O_NONBLOCK, NULL,
 		       data, data_len, amount);
-      mutex_unlock (&pipe->lock);
+      pthread_mutex_unlock (&pipe->lock);
     }
 
   return err;
@@ -556,9 +566,9 @@ trivfs_S_io_readable (struct trivfs_protid *cred,
     {
       struct pipe *pipe = cred->po->hook;
       assert (pipe);
-      mutex_lock (&pipe->lock);
+      pthread_mutex_lock (&pipe->lock);
       *amount = pipe_readable (pipe, 1);
-      mutex_unlock (&pipe->lock);
+      pthread_mutex_unlock (&pipe->lock);
       err = 0;
     }
 
@@ -585,10 +595,10 @@ trivfs_S_io_seek (struct trivfs_protid *cred,
    return the types that are then available.  ID_TAG is returned as passed; it
    is just for the convenience of the user in matching up reply messages with
    specific requests sent.  */
-error_t
-trivfs_S_io_select (struct trivfs_protid *cred,
-		    mach_port_t reply, mach_msg_type_name_t reply_type,
-		    int *select_type, int *tag)
+static error_t
+io_select_common (struct trivfs_protid *cred,
+		  mach_port_t reply, mach_msg_type_name_t reply_type,
+		  struct timespec *tsp, int *select_type)
 {
   struct pipe *pipe;
   error_t err = 0;
@@ -600,26 +610,43 @@ trivfs_S_io_select (struct trivfs_protid *cred,
   pipe = cred->po->hook;
 
   if (*select_type & SELECT_READ)
-    if (cred->po->openmodes & O_READ)
-      {
-	mutex_lock (&pipe->lock);
-	if (pipe_wait_readable (pipe, 1, 1) != EWOULDBLOCK)
-	  ready |= SELECT_READ; /* Data immediately readable (or error). */
-	mutex_unlock (&pipe->lock);
-      }
-    else
-      ready |= SELECT_READ;	/* Error immediately available...  */
+    {
+      if (cred->po->openmodes & O_READ)
+	{
+	  pthread_mutex_lock (&pipe->lock);
+	  err = pipe_wait_readable (pipe, 1, 1);
+	  if (err == EWOULDBLOCK)
+	    err = 0; /* Not readable, actually not an error.  */
+	  else
+	    ready |= SELECT_READ; /* Data immediately readable (or error).  */
+	  pthread_mutex_unlock (&pipe->lock);
+	}
+      else
+	{
+	  ready |= SELECT_READ;	/* Error immediately available...  */
+	}
+      if (err)
+	/* Prevent write test from overwriting err.  */
+	*select_type &= ~SELECT_WRITE;
+    }
 
   if (*select_type & SELECT_WRITE)
-    if (cred->po->openmodes & O_WRITE)
-      {
-	mutex_lock (&pipe->lock);
-	if (pipe_wait_writable (pipe, 1) != EWOULDBLOCK)
-	  ready |= SELECT_WRITE; /* Data immediately writable (or error). */
-	mutex_unlock (&pipe->lock);
-      }
-    else
-      ready |= SELECT_WRITE;	/* Error immediately available...  */
+    {
+      if (cred->po->openmodes & O_WRITE)
+	{
+	  pthread_mutex_lock (&pipe->lock);
+	  err = pipe_wait_writable (pipe, 1);
+	  if (err == EWOULDBLOCK)
+	    err = 0; /* Not writable, actually not an error.  */
+	  else
+	    ready |= SELECT_WRITE; /* Data immediately writable (or error).  */
+	  pthread_mutex_unlock (&pipe->lock);
+	}
+      else
+	{
+	  ready |= SELECT_WRITE;	/* Error immediately available...  */
+	}
+    }
 
   if (ready)
     *select_type = ready;
@@ -627,11 +654,28 @@ trivfs_S_io_select (struct trivfs_protid *cred,
     /* Wait for something to change.  */
     {
       ports_interrupt_self_on_port_death (cred, reply);
-      err = pipe_pair_select (pipe, pipe, select_type, 1);
+      err = pipe_pair_select (pipe, pipe, tsp, select_type, 1);
     }
 
   return err;
 }
+
+error_t
+trivfs_S_io_select (struct trivfs_protid *cred,
+		    mach_port_t reply, mach_msg_type_name_t reply_type,
+		    int *select_type)
+{
+  return io_select_common (cred, reply, reply_type, NULL, select_type);
+}
+
+error_t
+trivfs_S_io_select_timeout (struct trivfs_protid *cred,
+			    mach_port_t reply, mach_msg_type_name_t reply_type,
+			    struct timespec ts,
+			    int *select_type)
+{
+  return io_select_common (cred, reply, reply_type, &ts, select_type);
+}
 
 /* ---------------------------------------------------------------- */
 
@@ -657,10 +701,10 @@ trivfs_S_io_write (struct trivfs_protid *cred,
   else
     {
       struct pipe *pipe = cred->po->hook;
-      mutex_lock (&pipe->lock);
+      pthread_mutex_lock (&pipe->lock);
       err = pipe_write (pipe, cred->po->openmodes & O_NONBLOCK, NULL,
 			data, data_len, amount);
-      mutex_unlock (&pipe->lock);
+      pthread_mutex_unlock (&pipe->lock);
     }
 
   return err;
@@ -681,7 +725,7 @@ trivfs_S_file_set_size (struct trivfs_protid *cred,
 /* These four routines modify the O_APPEND, O_ASYNC, O_FSYNC, and
    O_NONBLOCK bits for the IO object. In addition, io_get_openmodes
    will tell you which of O_READ, O_WRITE, and O_EXEC the object can
-   be used for.  The O_ASYNC bit affects icky async I/O; good async 
+   be used for.  The O_ASYNC bit affects icky async I/O; good async
    I/O is done through io_async which is orthogonal to these calls. */
 
 error_t
