@@ -1,8 +1,7 @@
 /* Print task vm information
 
-   Copyright (C) 1996, 1997 Free Software Foundation, Inc.
-
-   Written by Miles Bader <miles@gnu.ai.mit.edu>
+   Copyright (C) 1996,97,98,2002 Free Software Foundation, Inc.
+   Written by Miles Bader <miles@gnu.org>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -143,7 +142,7 @@ main (int argc, char **argv)
 	  break;
 
 	case ARGP_KEY_NO_ARGS:
-	  argp_usage (state); 
+	  argp_usage (state);
 
 	default:
 	  return ARGP_ERR_UNKNOWN;
@@ -179,32 +178,40 @@ main (int argc, char **argv)
 	}
 
       if (holes && hole_addr != addr)
-	if ((what & (W_ADDRS|W_SIZES)) == (W_ADDRS|W_SIZES))
-	  if (hex)
-	    printf ("          [%#x] (hole)\n", addr - hole_addr);
-	  else
-	    printf ("          [%d] (hole)\n", addr - hole_addr);
-	else if ((what & (W_ADDRS|W_SIZES)) == W_SIZES)
-	  if (hex)
-	    printf ("%#10x (hole)\n", addr - hole_addr);
-	  else
-	    printf ("%10u (hole)\n", addr - hole_addr);
+	{
+	  if ((what & (W_ADDRS|W_SIZES)) == (W_ADDRS|W_SIZES))
+	    {
+	      if (hex)
+		printf ("          [%#lx] (hole)\n", addr - hole_addr);
+	      else
+		printf ("          [%lu] (hole)\n", addr - hole_addr);
+	    }
+	  else if ((what & (W_ADDRS|W_SIZES)) == W_SIZES)
+	    {
+	      if (hex)
+		printf ("%#10lx (hole)\n", addr - hole_addr);
+	      else
+		printf ("%10lu (hole)\n", addr - hole_addr);
+	    }
+	}
 
       if ((what & (W_ADDRS|W_SIZES)) == (W_ADDRS|W_SIZES))
 	if (hex)
-	  printf ("%#10x[%#x]", addr, size);
+	  printf ("%#10lx[%#zx]", addr, size);
 	else
-	  printf ("%10u[%d]", addr, size);
+	  printf ("%10lu[%zd]", addr, size);
       else if ((what & (W_ADDRS|W_SIZES)) == W_ADDRS)
 	if (hex)
-	  printf ("%#10x", addr);
+	  printf ("%#10lx", addr);
 	else
-	  printf ("%10u", addr);
+	  printf ("%10lu", addr);
       else if ((what & (W_ADDRS|W_SIZES)) == W_SIZES)
-	if (hex)
-	  printf ("%#10x", size);
-	else
-	  printf ("%10u", size);
+	{
+	  if (hex)
+	    printf ("%#10zx", size);
+	  else
+	    printf ("%10zu", size);
+	}
       if (what & W_DETAILS)
 	{
 	  printf (" (prot=%s", prot_rep (prot));
@@ -215,12 +222,14 @@ main (int argc, char **argv)
 	  if (shared)
 	    printf (", shared");
 	  if (obj != MACH_PORT_NULL)
-	    printf (", mem_obj=%d", obj);
+	    printf (", mem_obj=%lu", obj);
 	  if (offs != 0)
-	    if (hex)
-	      printf (", offs=%#x", offs);
-	    else
-	      printf (", offs=%d", offs);
+	    {
+	      if (hex)
+		printf (", offs=%#lx", offs);
+	      else
+		printf (", offs=%lu", offs);
+	    }
 	  putchar (')');
 	}
       putchar ('\n');
